@@ -1,5 +1,5 @@
 //----------------------------------*-C++-*----------------------------------//
-// $Id: plstream.cc,v 1.38 2006/05/13 06:35:05 airwin Exp $
+// $Id: plstream.cc 7891 2007-09-27 11:32:09Z andrewross $
 // Geoffrey Furnish
 // Sep 21 1994
 //
@@ -994,6 +994,16 @@ void plstream::mtex( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
     plmtex(side,disp,pos,just,text);
 }
 
+/* Prints out "text" at specified position relative to viewport (3D) */
+
+void plstream::mtex3( const char *side, PLFLT disp, PLFLT pos, PLFLT just,
+		     const char *text )
+{
+    set_stream();
+
+    plmtex3(side,disp,pos,just,text);
+}
+
 /* Plots a 3-d shaded representation of the function z[x][y]. */
 
 void plstream::surf3d( PLFLT *x, PLFLT *y, PLFLT **z,
@@ -1158,6 +1168,16 @@ void plstream::ptex( PLFLT x, PLFLT y, PLFLT dx, PLFLT dy, PLFLT just,
     plptex(x,y,dx,dy,just,text);
 }
 
+/* Prints out "text" at world cooordinate (x,y). */
+
+void plstream::ptex3( PLFLT wx, PLFLT wy, PLFLT wz, PLFLT dx, PLFLT dy, PLFLT dz, 
+		      PLFLT sx, PLFLT sy, PLFLT sz, PLFLT just, const char *text)
+{
+    set_stream();
+
+    plptex3(wx,wy,wz,dx,dy,dz,sx,sy,sz,just,text);
+}
+
 /* Replays contents of plot buffer to current device/file. */
 
 void plstream::replot()
@@ -1237,16 +1257,29 @@ void plstream::scmap1l( bool itype, PLINT npts, PLFLT *intensity,
 			PLFLT *coord1, PLFLT *coord2, PLFLT *coord3,
 			bool *rev )
 {
-    PLBOOL * loc_rev = new PLBOOL[npts-1];
-    for (int i=0;i<npts-1;i++) {
-        loc_rev[i] = (PLBOOL) rev[i];
+    PLBOOL * loc_rev = NULL;
+    if (rev != NULL) {
+        loc_rev = new PLBOOL[npts-1];
+        for (int i=0;i<npts-1;i++) {
+            loc_rev[i] = (PLBOOL) rev[i];
+	}
     }
     
     set_stream();
 
     plscmap1l((PLBOOL) itype,npts,intensity,coord1,coord2,coord3,loc_rev);
 
-    delete loc_rev;
+    if (loc_rev != NULL)
+        delete loc_rev;
+}
+
+void plstream::scmap1l( bool itype, PLINT npts, PLFLT *intensity,
+			PLFLT *coord1, PLFLT *coord2, PLFLT *coord3)
+{
+    set_stream();
+
+    plscmap1l((PLBOOL) itype,npts,intensity,coord1,coord2,coord3,NULL);
+
 }
 
 // Deprecated version using PLINT instead of bool
@@ -1254,16 +1287,20 @@ void plstream::scmap1l( PLINT itype, PLINT npts, PLFLT *intensity,
 			PLFLT *coord1, PLFLT *coord2, PLFLT *coord3,
 			PLINT *rev )
 {
-    PLBOOL * loc_rev = new PLBOOL[npts-1];
-    for (int i=0;i<npts-1;i++) {
-        loc_rev[i] = (PLBOOL) rev[i];
+    PLBOOL * loc_rev = NULL;
+    if (rev != NULL) {
+        loc_rev = new PLBOOL[npts-1];
+        for (int i=0;i<npts-1;i++) {
+            loc_rev[i] = (PLBOOL) rev[i];
+	}
     }
-
+    
     set_stream();
 
     plscmap1l((PLBOOL) itype,npts,intensity,coord1,coord2,coord3,loc_rev);
 
-    delete loc_rev;
+    if (loc_rev != NULL)
+        delete loc_rev;
 }
 
 /* Set a given color from color map 0 by 8 bit RGB value */
