@@ -1,4 +1,4 @@
-/* $Id: plstubs.h,v 1.4 2006/05/18 01:13:29 airwin Exp $
+/* $Id: plstubs.h 7910 2007-09-30 20:59:00Z airwin $
 
    Maurice LeBrun
    IFS, University of Texas
@@ -52,6 +52,7 @@
 #define STUB_L		2
 #define STUB_U		3
 #define STUB_FORTRAN	4
+#define STUB_WINFORTRAN	5
 
 #ifndef STUB_LINKAGE
 
@@ -89,6 +90,10 @@
 #endif
 #endif
 
+#ifdef WIN32				/* MS-DOS based */
+#define STUB_LINKAGE STUB_WINFORTRAN
+#endif
+
 #ifdef MSDOS				/* MS-DOS based */
 #define STUB_LINKAGE STUB_FORTRAN
 #endif
@@ -114,6 +119,9 @@
 #elif STUB_LINKAGE == STUB_U
 #define FNAME(x,y)	x
 
+#elif STUB_LINKAGE == STUB_WINFORTRAN
+#define FNAME(x,y)	PLDLLIMPEXP __stdcall x
+
 #elif STUB_LINKAGE == STUB_FORTRAN
 #define FNAME(x,y)	fortran x
 
@@ -207,8 +215,16 @@
 #define    PLMESHC	FNAME(PLMESHCF77,plmeshcf77)
 #define    PLMKSTRM	FNAME(PLMKSTRM,plmkstrm)
 #define    PLMTEX7	FNAME(PLMTEX7,plmtex7)
+#define    PLMTEX37	FNAME(PLMTEX37,plmtex37)
 #define    PLOT3D	FNAME(PLOT3DF77,plot3df77)
 #define    PLOT3DC	FNAME(PLOT3DCF77,plot3dcf77)
+
+#if STUB_LINKAGE == STUB_WINFORTRAN || STUB_LINKAGE == STUB_FORTRAN
+#define    CALL_PLOT3DC PLOT3DCF77
+#else
+#define    CALL_PLOT3DC PLOT3DC
+#endif
+
 #define    PLPARSEOPTS7	FNAME(PLPARSEOPTS7,plparseopts7)
 #define    PLPAT	FNAME(PLPAT,plpat)
 #define    PLPOIN	FNAME(PLPOINF77,plpoinf77)
@@ -217,6 +233,7 @@
 #define    PLPREC	FNAME(PLPREC,plprec)
 #define    PLPSTY	FNAME(PLPSTY,plpsty)
 #define    PLPTEX7	FNAME(PLPTEX7,plptex7)
+#define    PLPTEX37	FNAME(PLPTEX37,plptex37)
 #define    PLREPLOT	FNAME(PLREPLOT,plreplot)
 #define    PLRGB	FNAME(PLRGB,plrgb)
 #define    PLRGB1	FNAME(PLRGB1,plrgb1)
@@ -226,6 +243,7 @@
 #define    PLSCMAP0N	FNAME(PLSCMAP0N,plscmap0n)
 #define    PLSCMAP1	FNAME(PLSCMAP1F77,plscmap1f77)
 #define    PLSCMAP1L	FNAME(PLSCMAP1LF77,plscmap1lf77)
+#define    PLSCMAP1LA	FNAME(PLSCMAP1LAF77,plscmap1laf77)
 #define    PLSCMAP1N	FNAME(PLSCMAP1N,plscmap1n)
 #define    PLSCOL0	FNAME(PLSCOL0,plscol0)
 #define    PLSCOLBG	FNAME(PLSCOLBG,plscolbg)
