@@ -1,4 +1,4 @@
-/* $Id: sccont.c,v 1.3 2006/05/19 10:09:44 arjenmarkus Exp $
+/* $Id: sccont.c 7112 2006-10-23 19:56:05Z arjenmarkus $
 
 	Contour plotter front-ends for Fortran.
 
@@ -723,7 +723,7 @@ PLSHADES7(PLFLT *z, PLINT *nx, PLINT *ny, char *defined,
 }
 
 void
-PLGRIDDATAF77(PLFLT *x, PLFLT *y, PLFLT *z, PLINT *npts, PLFLT *xg,
+PLGRIDDATA(PLFLT *x, PLFLT *y, PLFLT *z, PLINT *npts, PLFLT *xg,
 	 PLINT *nx, PLFLT *yg, PLINT *ny, PLFLT *zg, PLINT *type, PLFLT *data )
 {
    PLFLT ** a;
