@@ -1,4 +1,4 @@
-/* $Id: scstubs.c,v 1.7 2006/05/22 22:43:18 airwin Exp $
+/* $Id: scstubs.c 7910 2007-09-30 20:59:00Z airwin $
 
 	C stub routines.
 
@@ -240,7 +240,7 @@ PLGCOMPRESSION(PLINT *compression)
 }
 
 void
-PLGDEV7(char *dev)
+PLGDEV7(char *dev, int length )
 {
     c_plgdev(dev);
 }
@@ -270,7 +270,7 @@ PLGFAM(PLINT *fam, PLINT *num, PLINT *bmax)
 }
 
 void
-PLGFNAM7(char *fnam)
+PLGFNAM7(char *fnam, int length)
 {
     c_plgfnam(fnam);
 }
@@ -294,6 +294,7 @@ PLGRA(void)
     c_plgra();
 }
 
+/* AM - removed: also defined, and better in sccont.c
 void
 PLGRIDDATA(PLFLT *x, PLFLT *y, PLFLT *z, PLINT *npts,
 	   PLFLT *xg, PLINT *nptsx, PLFLT *yg,  PLINT *nptsy,
@@ -303,6 +304,7 @@ PLGRIDDATA(PLFLT *x, PLFLT *y, PLFLT *z, PLINT *npts,
 		 xg, *nptsx, yg, *nptsy,
 		 zg, *type, *data);
 }
+*/
 
 void
 PLGSPA(PLFLT *xmin, PLFLT *xmax, PLFLT *ymin, PLFLT *ymax)
@@ -447,6 +449,12 @@ PLMTEX7(char *side, PLFLT *disp, PLFLT *pos, PLFLT *just, char *text)
     c_plmtex(side, *disp, *pos, *just, text);
 }
 
+void
+PLMTEX37(char *side, PLFLT *disp, PLFLT *pos, PLFLT *just, char *text)
+{
+    c_plmtex3(side, *disp, *pos, *just, text);
+}
+
 void
 PLPARSEOPTS7(int *numargs, char *iargs, PLINT *mode, PLINT *maxindex)
 {
@@ -506,6 +514,16 @@ PLPTEX7(PLFLT *x, PLFLT *y, PLFLT *dx, PLFLT *dy, PLFLT *just, char *text)
     c_plptex(*x, *y, *dx, *dy, *just, text);
 }
 
+void
+PLPTEX37(
+	 PLFLT *x, PLFLT *y, PLFLT *z, 
+	 PLFLT *dx, PLFLT *dy, PLFLT *dz,
+	 PLFLT *sx, PLFLT *sy, PLFLT *sz,
+	 PLFLT *just, char *text)
+{
+    c_plptex3(*x, *y, *z, *dx, *dy, *dz, *sx, *sy, *sz, *just, text);
+}
+
 void
 PLREPLOT(void)
 {
@@ -561,6 +579,13 @@ PLSCMAP1L(PLBOOL *itype, PLINT *npts, PLFLT *intensity,
     c_plscmap1l(*itype, *npts, intensity, coord1, coord2, coord3, rev);
 }
 
+void
+PLSCMAP1LA(PLBOOL *itype, PLINT *npts, PLFLT *intensity,
+	  PLFLT *coord1, PLFLT *coord2, PLFLT *coord3)
+{
+    c_plscmap1l(*itype, *npts, intensity, coord1, coord2, coord3, NULL);
+}
+
 void
 PLSCMAP1N(PLINT *n)
 {
