@@ -1,5 +1,5 @@
 !***********************************************************************
-!  $Id: sfstubsf95.f90,v 1.6 2006/05/19 17:15:15 airwin Exp $
+!  $Id: sfstubsf95.f90 7831 2007-09-04 19:41:53Z arjenmarkus $
 !  sfstubsf95.f
 !
 !  Copyright (C) 2005, 2006  Arjen Markus
@@ -540,6 +540,7 @@
 
       interface plscmap1l
          module procedure plscmap1l
+         module procedure plscmap1la
       end interface
 
       interface
@@ -1024,6 +1025,16 @@
          call plscmap1lf77( type, size(intensity), intensity, coord1, coord2, coord3, irev )
       end subroutine plscmap1l
 
+      subroutine plscmap1la( rgbtype, intensity, coord1, coord2, coord3)
+            logical                        :: rgbtype
+            real(kind=plflt), dimension(:) :: intensity, coord1, coord2, coord3
+
+            integer                        :: type
+
+         type = convert_to_int( rgbtype )
+         call plscmap1laf77( type, size(intensity), intensity, coord1, coord2, coord3)
+      end subroutine plscmap1la
+
       subroutine plstripc(id, xspec, yspec, xmin, xmax, xjump, &
         ymin, ymax, xlpos, ylpos, y_ascl, acc, &
         colbox, collab, colline, styline, legline, &
@@ -1052,6 +1063,15 @@
       iy_ascl = convert_to_int( y_ascl )
       iacc = convert_to_int( acc )
 
+      s1 = transfer( string1, s1 )
+      s2 = transfer( string2, s2 )
+      s3 = transfer( string3, s3 )
+      s4 = transfer( string4, s4 )
+      s5 = transfer( string5, s5 )
+      s6 = transfer( string6, s6 )
+      s7 = transfer( string7, s7 )
+      s8 = transfer( string8, s8 )
+      s9 = transfer( string9, s9 )
       call plstripcf77(id, s1, s2, xmin, xmax, xjump, &
         ymin, ymax, xlpos, ylpos, iy_ascl, iacc, &
         colbox, collab, colline, styline, &
