@@ -174,9 +174,9 @@ public void cont(double[][] f, int kx, int lx, int ky, int ly,
     plplotjavac.plcont(f, kx, lx, ky, ly, clevel, pltr, OBJECT_DATA);
 }
 
-public void cpstrm(int iplsr, boolean flags) {
+public void cpstrm(PLStream pls, boolean flags) {
     if (set_stream() == -1) return;
-    plplotjavac.plcpstrm(iplsr, flags);
+    plplotjavac.plcpstrm(pls.stream_id, flags);
 }
 
 // The end / end1 functions have extra code in to keep track of the
@@ -201,6 +201,11 @@ public void env(double xmin, double xmax, double ymin, double ymax, int just, in
     plplotjavac.plenv(xmin, xmax, ymin, ymax, just, axis);
 }
 
+public void env0(double xmin, double xmax, double ymin, double ymax, int just, int axis) {
+    if (set_stream() == -1) return;
+    plplotjavac.plenv0(xmin, xmax, ymin, ymax, just, axis);
+}
+
 public void eop() {
     if (set_stream() == -1) return;
     plplotjavac.pleop();
@@ -286,6 +291,11 @@ public void gdiplt(double[] xmin, double[] xmax, double[] ymin, double[] ymax) {
     plplotjavac.plgdiplt(xmin, xmax, ymin, ymax);
 }
 
+public int getCursor(PLGraphicsIn gin) {
+    if (set_stream() == -1) return 0;
+    return plplotjavac.plGetCursor(gin);
+}
+
 public void gfam(int[] fam, int[] num, int[]  bmax) {
     if (set_stream() == -1) return;
     plplotjavac.plgfam(fam, num, bmax);
@@ -311,6 +321,12 @@ public void gra() {
     plplotjavac.plgra();
 }
 
+public void griddata(double[] x, double[] y, double[] z, double[] xg, 
+		     double[] yg, double[][] zg, int type, double data) {
+    if (set_stream() == -1) return;
+    plplotjavac.plgriddata(x, y, z, xg, yg, zg, type, data);
+}
+
 public void gspa(double[] xmin, double[] xmax, double[] ymin, double[] ymax) {
     if (set_stream() == -1) return;
     plplotjavac.plgspa(xmin, xmax, ymin, ymax);
@@ -363,6 +379,11 @@ public void hls(double h, double l, double s) {
     plplotjavac.plhls(h, l, s);
 }
 
+public void image(double[][] data, double xmin, double xmax, double ymin, double ymax, double zmin, double zmax, double Dxmin, double Dxmax, double Dymin, double Dymax) {
+    if (set_stream() == -1) return;
+    plplotjavac.plimage(data, xmin, xmax, ymin, ymax, zmin, zmax, Dxmin, Dxmax, Dymin, Dymax);
+}
+
 public void init() {
     if (set_stream() == -1) return;
     plplotjavac.plinit();
@@ -398,6 +419,12 @@ public void lsty(int lin) {
     plplotjavac.pllsty(lin);
 }
 
+public void minMax2dGrid(double[][]f, double[] fmax, double[] fmin) {
+    if (set_stream() == -1) return;
+    plplotjavac.plMinMax2dGrid(f,fmax,fmin);
+    
+}
+
 public void mesh(double[] x, double[] y, double[][] z, int opt) {
     if (set_stream() == -1) return;
     plplotjavac.plmesh(x, y, z, opt);
@@ -419,6 +446,11 @@ public void mtex(String side, double disp, double pos, double just, String text)
     plplotjavac.plmtex(side, disp, pos, just, text);
 }
 
+public void mtex3(String side, double disp, double pos, double just, String text) {
+    if (set_stream() == -1) return;
+    plplotjavac.plmtex3(side, disp, pos, just, text);
+}
+
 public void plot3d(double[] x, double[] y, double[][] z, int opt, boolean side) {
     if (set_stream() == -1) return;
     plplotjavac.plot3d(x, y, z, opt, side);
@@ -486,6 +518,11 @@ public void ptex(double x, double y, double dx, double dy, double just, String t
     plplotjavac.plptex(x, y, dx, dy, just, text);
 }
 
+public void ptex3(double x, double y, double z, double dx, double dy, double dz, double sx, double sy, double sz, double just, String text) {
+    if (set_stream() == -1) return;
+    plplotjavac.plptex3(x, y, z, dx, dy, dz, sx, sy, sz, just, text);
+}
+
 public void replot() {
     if (set_stream() == -1) return;
     plplotjavac.plreplot();
@@ -517,6 +554,12 @@ public void scmap1l(boolean itype, double[] intensity, double[] coord1,
     plplotjavac.plscmap1l(itype, intensity, coord1, coord2, coord3, rev);
 }
 
+public void scmap1l(boolean itype, double[] intensity, double[] coord1, 
+		double[] coord2, double[] coord3) {
+    if (set_stream() == -1) return;
+    plplotjavac.plscmap1l(itype, intensity, coord1, coord2, coord3, null);
+}
+
 public void scmap1n(int ncol1) {
     if (set_stream() == -1) return;
     plplotjavac.plscmap1n(ncol1);
@@ -672,7 +715,7 @@ public void stripc(int[] id, String xspec, String yspec,
 		double xmin, double xmax, double xjump, 
 		double ymin, double ymax, double xlpos, double ylpos, 
 		boolean y_ascl, boolean acc, int colbox, int collab, 
-		int[] colline, int[] styline, SWIGTYPE_p_p_char legline, 
+		int[] colline, int[] styline, String[] legline, 
 		String labx, String laby, String labtop) {
     if (set_stream() == -1) return;
     plplotjavac.plstripc(id, xspec, yspec, xmin, xmax, xjump, ymin, ymax, 
@@ -827,9 +870,12 @@ public void poly3(double[] x, double[] y, double[] z, int[] draw, int ifcc) {
 public void scmap1l(int itype, double[] intensity, double[] coord1, 
 		double[] coord2, double[] coord3, int[] rev) {
     if (set_stream() == -1) return;
-    boolean [] loc_rev = new boolean[rev.length];
-    for (int i=0;i<rev.length;i++) {
-	loc_rev[i] = (rev[i]!=0);
+    boolean [] loc_rev = null;
+    if (rev != null) {
+        loc_rev = new boolean[rev.length];
+        for (int i=0;i<rev.length;i++) {
+	    loc_rev[i] = (rev[i]!=0);
+        }
     }
     plplotjavac.plscmap1l(itype != 0, intensity, coord1, coord2, coord3, loc_rev);
 }
@@ -863,7 +909,7 @@ public void stripc(int[] id, String xspec, String yspec,
 		double xmin, double xmax, double xjump, 
 		double ymin, double ymax, double xlpos, double ylpos, 
 		int y_ascl, int acc, int colbox, int collab, 
-		int[] colline, int[] styline, SWIGTYPE_p_p_char legline, 
+		int[] colline, int[] styline, String[] legline, 
 		String labx, String laby, String labtop) {
     if (set_stream() == -1) return;
     plplotjavac.plstripc(id, xspec, yspec, xmin, xmax, xjump, ymin, ymax, 
