@@ -1,4 +1,4 @@
-/* $Id: matrixInit.c,v 1.9 2005/04/27 06:43:20 rlaboiss Exp $
+/* $Id: matrixInit.c 7082 2006-09-18 04:36:21Z arjenmarkus $
 
     Tcl Matrix initializer.
     Vince Darley
@@ -23,9 +23,10 @@
 
 */
 
+#include "pldll.h"
 #include "tclMatrix.h"
 
-int Matrix_Init( Tcl_Interp *interp ) {
+int PLDLLIMPEXP Matrix_Init( Tcl_Interp *interp ) {
 
     /* matrix -- matrix	support	command	*/
     Tcl_CreateCommand(interp, "matrix",	(Tcl_CmdProc *) Tcl_MatrixCmd,
