@@ -1,4 +1,4 @@
-/* $Id: pltcl.h,v 1.10 2002/12/03 08:39:22 airwin Exp $
+/* $Id: pltcl.h 7083 2006-09-19 04:47:04Z arjenmarkus $
  *
  * Maurice LeBrun
  * 21-Jun-94
@@ -11,6 +11,7 @@
 #define __PLTCL_H__
 
 #include "plplot.h"
+#include "pldll.h"
 #include <tcl.h>
 #include "tclMatrix.h"
 
@@ -21,7 +22,7 @@ extern "C" {
 /* tclMain.c */
 /* Main program for a Tcl-based shell that reads Tcl commands from stdin. */
 
-int
+int PLDLLIMPEXP
 pltclMain(int argc, char **argv, char *RcFileName,
 	  int (*AppInit)(Tcl_Interp *interp));
 
@@ -34,7 +35,7 @@ plTclCmd(char *cmdlist, Tcl_Interp *interp,
 
 /* Initialization routine for PLplot-extended tclsh's (like pltcl). */
 
-int
+int PLDLLIMPEXP
 Pltcl_Init( Tcl_Interp *interp );
 
 int
