@@ -1,4 +1,4 @@
-/* $Id: tclAPI.c,v 1.60 2005/04/27 06:43:20 rlaboiss Exp $
+/* $Id: tclAPI.c 7206 2006-11-27 21:07:16Z smekal $
 
     Copyright 1994, 1995
     Maurice LeBrun			mjl@dino.ph.utexas.edu
@@ -31,12 +31,18 @@
     driver, in principle.
 */
 
+#define USINGPLDLL
 #include "plplotP.h"
 #include "pltcl.h"
 #ifndef __WIN32__
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#else
+#ifdef _MSC_VER
+#define getcwd _getcwd
+#include <direct.h>
+#endif
 #endif
 
 #include "tclgen.h"
@@ -73,7 +79,7 @@ typedef struct Command {
 
 typedef struct {
     char *name;
-    int (*proc)();
+    int (*proc)(void *, struct Tcl_Interp *, int, char **);
 } CmdInfo;
 
 /* Built-in commands, and the procedures associated with them */
@@ -104,7 +110,7 @@ static Tcl_HashTable cmdTable;
 
 /* Variables for holding error return info from PLplot */
 
-static PLINT errcode;
+static PLINT pl_errcode;
 static char errmsg[160];
 
 /* Library initialization */
@@ -113,7 +119,8 @@ static char errmsg[160];
 #define PL_LIBRARY ""
 #endif
 
-extern char* plplotLibDir;
+/* extern char PLDLLIMPEXP * plplotLibDir; */
+extern char PLDLLIMPORT * plplotLibDir;
 
 #if (!defined(MAC_TCL) && !defined(__WIN32__))
 /*
@@ -185,7 +192,7 @@ plTclCmd_Init(Tcl_Interp *interp)
 
 /* Register our error variables with PLplot */
 
-    plsError(&errcode, errmsg);
+    plsError(&pl_errcode, errmsg);
 
 /* Initialize hash table */
 
@@ -236,7 +243,7 @@ plTclCmd(char *cmdlist, Tcl_Interp *interp, int argc, char **argv)
     register Tcl_HashEntry *hPtr;
     int result = TCL_OK;
 
-    errcode = 0; errmsg[0] = '\0';
+    pl_errcode = 0; errmsg[0] = '\0';
 
 /* Create hash table on first call */
 
@@ -267,7 +274,7 @@ plTclCmd(char *cmdlist, Tcl_Interp *interp, int argc, char **argv)
 	register Command *cmdPtr = (Command *) Tcl_GetHashValue(hPtr);
 	result = (*cmdPtr->proc)(cmdPtr->clientData, interp, argc, argv);
 	if (result == TCL_OK) {
-	    if (errcode != 0) {
+	    if (pl_errcode != 0) {
 		result = TCL_ERROR;
 		Tcl_AppendResult(interp, errmsg, (char *) NULL);
 	    }
@@ -503,7 +510,7 @@ PlbasicInit( Tcl_Interp *interp )
  * interfacing to PLplot.  Should not be used in a widget-based system.
 \*--------------------------------------------------------------------------*/
 
-int
+int PLDLLEXPORT
 Pltcl_Init( Tcl_Interp *interp )
 {
     register CmdInfo *cmdInfoPtr;
@@ -514,7 +521,7 @@ Pltcl_Init( Tcl_Interp *interp )
 
 /* Register our error variables with PLplot */
 
-    plsError(&errcode, errmsg);
+    plsError(&pl_errcode, errmsg);
 
 /* PLplot API commands */
 
