@@ -1,4 +1,4 @@
-/* $Id: tclMain.c,v 1.19 2005/04/27 06:43:20 rlaboiss Exp $
+/* $Id: tclMain.c 7083 2006-09-19 04:47:04Z arjenmarkus $
  *
  * Modified version of tclMain.c, from Tcl 8.3.2.
  * Maurice LeBrun
@@ -45,11 +45,13 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclMain.c,v 1.19 2005/04/27 06:43:20 rlaboiss Exp $
+ * RCS: @(#) $Id: tclMain.c 7083 2006-09-19 04:47:04Z arjenmarkus $
  */
 
 #include <tcl.h>
 #include "plplot.h"
+#define MAKINGPLDLL
+#include "pldll.h"
 
 /* From tclIntDecls.h */
 
@@ -205,7 +207,7 @@ char *TclGetStartupScriptFileName(void)
  *----------------------------------------------------------------------
  */
 
-int
+int PLDLLEXPORT
 pltclMain(int argc, char **argv, char *RcFileName /* OBSOLETE */,
 	  int (*appInitProc)(Tcl_Interp *interp))
 {
