@@ -1,4 +1,4 @@
-/* $Id: tclMatrix.c,v 1.25 2005/04/27 06:43:20 rlaboiss Exp $
+/* $Id: tclMatrix.c 7082 2006-09-18 04:36:21Z arjenmarkus $
 
     Copyright 1994, 1995
     Maurice LeBrun			mjl@dino.ph.utexas.edu
@@ -38,6 +38,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "pldll.h"
 #include "tclMatrix.h"
 
 /* Cool math macros */
@@ -348,7 +349,7 @@ Tcl_MatrixCmd(ClientData clientData, Tcl_Interp *interp,
  *
 \*--------------------------------------------------------------------------*/
 
-tclMatrix *
+tclMatrix PLDLLIMPEXP *
 Tcl_GetMatrixPtr(Tcl_Interp *interp, char *matName)
 {
     Tcl_HashEntry *hPtr;
