@@ -177,12 +177,11 @@ static inline void NOOP_(id x, ...) {;}
 
 /* helper functions */
 
-void get_cursor(PLStream *, PLGraphicsIn *);
-void proc_str (PLStream *, EscText *);
+static void get_cursor(PLStream *, PLGraphicsIn *);
+static void proc_str (PLStream *, EscText *);
 NSMutableAttributedString * create_string(const PLUNICODE *, int, PLFLT);
-void set_font_and_size(NSMutableAttributedString *, PLUNICODE, PLFLT, int);
-//char * UCS4_to_UTF8(const PLUNICODE);
-void check_font_environment_variables(void);
+static void set_font_and_size(NSMutableAttributedString *, PLUNICODE, PLFLT, int);
+static void check_font_environment_variables(void);
 
 /* PLplot interface functions */
 
@@ -237,6 +236,7 @@ void plD_init_aqt(PLStream *pls)
    [adapter setBackgroundColorRed:0.5 green:0.5 blue:0.5];
 
    pls->termin = 1;			/* interactive device */
+   pls->dev_flush = 1;		/* Handle our own flushes */
    pls->color = 1;			/* supports color */
    pls->width = 1;
    pls->verbose = 1;
@@ -428,6 +428,7 @@ void plD_esc_aqt(PLStream *pls, PLINT op, void *ptr)
       case PLESC_DI:                  // handle DI command
          break;
       case PLESC_FLUSH:               // flush output
+         [adapter renderPlot];
          break;
       case PLESC_EH:                  // handle Window events
          break;
@@ -483,7 +484,6 @@ void get_cursor(PLStream *pls, PLGraphicsIn *gin){
 
 void proc_str (PLStream *pls, EscText *args)
 {
-	PLFLT   	*t = args->xform;
 	PLFLT   	a1, ft_ht, angle, shear;
 	PLINT   	clxmin, clxmax, clymin, clymax;
 	int     	i, jst, ref;
@@ -508,14 +508,9 @@ void proc_str (PLStream *pls, EscText *args)
 	ft_ht = 1.2 * pls->chrht * DPI/25.4; 	/* ft_ht in points. ht is in mm */
 
    	/* given transform, calculate rotation angle & shear angle */
-	
-	a1 = acos(t[0]) * 180. / PI;
-	if (t[2] > 0.)
-		angle = a1;
-	else
-		angle = 360. - a1;
-
-	shear = 90.0 - acos(t[0]*t[1] + t[2]*t[3]) * 180.0 / PI;
+    plRotationShear(args->xform, &angle, &shear);
+    angle *= 180.0/PI;
+    shear *= -180.0/PI;
 
 	/* text justification, AquaTerm only supports 3 options, so we round appropriately */
 	
@@ -527,14 +522,16 @@ void proc_str (PLStream *pls, EscText *args)
 		jst = AQTAlignCenter;                           /* center */
 	
 	/* set the baseline of the string */
+	// Middle and Bottom are set to Middle since this seems to be what PLplot expects
+	// as judged by where it renders the symbols in example 1.
 	
-	if (args->base == 2)
+	if (args->base == 2)      // Top
 		ref = AQTAlignTop;
-	else if (args->base == 1)
-		ref = AQTAlignBaseline;
-	else
+	else if (args->base == 1) // Bottom
 		ref = AQTAlignMiddle;
-
+	else
+		ref = AQTAlignMiddle; // Middle
+	
 	/* create an appropriately formatted, etc... unicode string */
 	
 	str = create_string(args->unicode_array, args->unicode_array_len, ft_ht);
