@@ -1,4 +1,4 @@
-/* $Id: gd.c,v 1.43 2006/05/18 11:56:39 rlaboiss Exp $
+/* $Id: gd.c 7877 2007-09-20 04:19:31Z aroach $
 
          PNG, GIF, and JPEG device driver based on libgd
 
@@ -27,8 +27,8 @@
  *  Following the expiration of Unisys's worldwide patents on lzw compression
  *  GD 2.0.28+ have reinstated support for GIFs, and so support for this
  *  format has been added to the GD family of drivers. GIF's only support
- *  1, 4 and 8 bit, so no truecolour. Why would you want GIFs though ? PNG is 
- *  a far superior format, not only giving you 1,4,8 and 24 bit, but also 
+ *  1, 4 and 8 bit, so no truecolour. Why would you want GIFs though ? PNG is
+ *  a far superior format, not only giving you 1,4,8 and 24 bit, but also
  *  better compression and just about all browsers now support them.
  */
 
@@ -108,6 +108,11 @@
  *  way of making a nice web-friendly png without having to redefine the
  *  cmaps within your program.
  *
+ *  smoothlines
+ *
+ *  -drvopt smoothlines=1 turns on anti-aliased line and polygong drawing if
+ *  you are using a 24bit mode. Unfortunately gd doesn't honour line
+ *  width when anti-aliasing, so by default it is off.
  */
 
 
@@ -127,7 +132,7 @@
  *  and finally one of each.
  */
 
-char* plD_DEVICE_INFO_gd = 
+char* plD_DEVICE_INFO_gd =
 #if defined(PLD_png)
   "png:PNG file:0:gd:39:png\n"
 #endif
@@ -139,6 +144,11 @@ char* plD_DEVICE_INFO_gd =
 #endif
 ;
 
+#if GD2_VERS >= 2
+#ifdef HAVE_FREETYPE
+#define SMOOTH_LINES_OK
+#endif
+#endif
 
 #ifdef HAVE_FREETYPE
 
@@ -178,6 +188,8 @@ static void     plD_init_gif_Dev(PLStream *pls);
 #ifdef HAVE_FREETYPE
 
 static void plD_pixel_gd (PLStream *pls, short x, short y);
+static PLINT plD_read_pixel_gd (PLStream *pls, short x, short y);
+static void plD_set_pixel_gd (PLStream *pls, short x, short y, PLINT colour);
 static void init_freetype_lv1 (PLStream *pls);
 static void init_freetype_lv2 (PLStream *pls);
 
@@ -242,9 +254,12 @@ typedef struct {
         int black15;                            /* Flag used for forcing a black colour */
         int red15;                              /* Flag for swapping red and 15 */
 
+        unsigned char TRY_BLENDED_ANTIALIASING;  /* Flag to try and set up BLENDED ANTIALIASING */
+
 #if GD2_VERS >= 2
         int truecolour;                         /* Flag to ALWAYS force 24 bit mode */
         int palette;                            /* Flag to ALWAYS force  8 bit mode */
+        unsigned char smooth;                   /* Flag to ask for line smoothing */
 #endif
 
 } png_Dev;
@@ -349,6 +364,7 @@ plD_init_png_Dev(PLStream *pls)
 #if GD2_VERS >= 2
     static int truecolour=0;
     static int palette=0;
+    static int smooth_line=0;
 #endif
 #ifdef HAVE_FREETYPE
     static int freetype=1;
@@ -362,6 +378,7 @@ plD_init_png_Dev(PLStream *pls)
 #if GD2_VERS >= 2
                               {"8bit", DRV_INT, &palette, "Palette (8 bit) mode"},
                               {"24bit", DRV_INT, &truecolour, "Truecolor (24 bit) mode"},
+                              {"smoothlines", DRV_INT, &smooth_line, "Turn line Anti Aliasing on (1) or off (0)"},
 #endif
 #ifdef HAVE_FREETYPE
                               {"text", DRV_INT, &freetype, "Use driver text (FreeType)"},
@@ -403,6 +420,8 @@ plD_init_png_Dev(PLStream *pls)
     dev->palette=palette;
     dev->truecolour=truecolour;
 
+
+
     if ((dev->truecolour>0) && (dev->palette>0))
        plwarn("Selecting both \"truecolor\" AND \"palette\" driver options is contradictory, so\nI will just use my best judgment.\n");
     else if (dev->truecolour>0)
@@ -412,6 +431,8 @@ plD_init_png_Dev(PLStream *pls)
         NCOLOURS=16777216;
        }
 
+    if ((dev->palette==0)&&(dev->optimise==0)&&(smooth_line==1)) dev->smooth=1; /* Allow smoothing of lines if we have a truecolour device */
+
 #endif
 
 #ifdef HAVE_FREETYPE
@@ -420,9 +441,20 @@ if (freetype)
     pls->dev_text = 1; /* want to draw text */
     pls->dev_unicode = 1; /* want unicode */
 
+    /* As long as we aren't optimising, we'll try to use better antialaising
+     * We can also only do this if the user wants smoothing, and hasn't
+     * selected a palette mode.
+     */
+
+
     init_freetype_lv1(pls);
     FT=(FT_Data *)pls->FT;
-    FT->want_smooth_text=smooth_text;
+    FT->want_smooth_text=smooth_text>0 ? 1 : 0;
+    if ((dev->optimise==0)&&(dev->palette==0)&&(smooth_text!=0))
+      {
+        FT->BLENDED_ANTIALIASING=1;
+        dev->truecolour=1;
+      }
    }
 
 #endif
@@ -577,7 +609,8 @@ if (freetype)
 
     init_freetype_lv1(pls);
     FT=(FT_Data *)pls->FT;
-    FT->want_smooth_text=smooth_text;
+
+    FT->want_smooth_text=smooth_text > 0 ? 1 : 0;
    }
 
 #endif
@@ -681,7 +714,19 @@ plD_line_png(PLStream *pls, short x1a, short y1a, short x2a, short y2a)
     y1 = dev->pngy - y1;
     y2 = dev->pngy - y2;
 
-    gdImageLine(dev->im_out, x1, y1, x2, y2, dev->colour);
+    #ifdef SMOOTH_LINES_OK
+      if (dev->smooth==1)
+        {
+          gdImageSetAntiAliased(dev->im_out,dev->colour);
+          gdImageLine(dev->im_out, x1, y1, x2, y2, gdAntiAliased);
+        }
+      else
+        {
+          gdImageLine(dev->im_out, x1, y1, x2, y2, dev->colour);
+        }
+    #else
+      gdImageLine(dev->im_out, x1, y1, x2, y2, dev->colour);
+    #endif
 
 }
 
@@ -726,7 +771,20 @@ png_Dev *dev=(png_Dev *)pls->dev;
 	   points[i].y = dev->pngy - (pls->dev_y[i]/dev->scale);
          }
 
-   gdImageFilledPolygon(dev->im_out, points, pls->dev_npts, dev->colour);
+    #ifdef SMOOTH_LINES_OK
+      if (dev->smooth==1)
+        {
+          gdImageSetAntiAliased(dev->im_out,dev->colour);
+          gdImageFilledPolygon(dev->im_out, points, pls->dev_npts, gdAntiAliased);
+        }
+      else
+        {
+          gdImageFilledPolygon(dev->im_out, points, pls->dev_npts, dev->colour);
+        }
+    #else
+      gdImageFilledPolygon(dev->im_out, points, pls->dev_npts, dev->colour);
+    #endif
+
    free(points);
 
 }
@@ -1025,6 +1083,7 @@ if (dev->red15) plD_red15_gd(pls);
        else
          {
          dev->im_out = gdImageCreateTrueColor(pls->xlength, pls->ylength);
+         plP_state(PLSTATE_COLOR0);
 
 /*
  * In truecolour mode, the background colour GD makes is ALWAYS black, so to
@@ -1215,7 +1274,9 @@ free(bbuf);
 
 void plD_eop_png(PLStream *pls)
 {
-png_Dev *dev=(png_Dev *)pls->dev;
+    png_Dev *dev=(png_Dev *)pls->dev;
+    int im_size=0;
+    void *im_ptr=NULL;
 
     if (pls->family || pls->page == 1) {
 
@@ -1235,7 +1296,20 @@ png_Dev *dev=(png_Dev *)pls->dev;
 #endif
         }
 
-       gdImagePng(dev->im_out, pls->OutFile);
+
+       /* image is written to output file by the driver
+          since if the gd.dll is linked to a different c
+          lib a crash occurs - this fix works also in Linux */
+       /* gdImagePng(dev->im_out, pls->OutFile); */
+       #if GD2_VERS >= 2
+         im_ptr = gdImagePngPtrEx (dev->im_out, &im_size, pls->dev_compression >9 ? (pls->dev_compression/10) : pls->dev_compression);
+       #else
+         im_ptr = gdImagePngPtr(dev->im_out, &im_size);
+       #endif
+       if( im_ptr ) {
+         fwrite(im_ptr, sizeof(char), im_size, pls->OutFile);
+         gdFree(im_ptr);
+       }
 
        gdImageDestroy(dev->im_out);
        dev->im_out = NULL;
@@ -1261,6 +1335,52 @@ png_Dev *dev=(png_Dev *)pls->dev;
 
 }
 
+/*----------------------------------------------------------------------*\
+ *  void plD_set_pixel_gd (PLStream *pls, short x, short y)
+ *
+ *  callback function, of type "plD_pixel_fp", which specifies how a single
+ *  pixel is set directly to hardware, using the colour provided
+\*----------------------------------------------------------------------*/
+
+void plD_set_pixel_gd (PLStream *pls, short x, short y, PLINT colour)
+{
+png_Dev *dev=(png_Dev *)pls->dev;
+int R,G,B;
+int Colour;
+
+   G=GetGValue(colour);
+   R=GetRValue(colour);
+   B=GetBValue(colour);
+
+   Colour=gdImageColorResolve(dev->im_out,R,G,B);
+   gdImageSetPixel(dev->im_out, x, y,Colour);
+}
+
+/*----------------------------------------------------------------------*\
+ *  PLINT plD_read_pixel_gd (PLStream *pls, short x, short y)
+ *
+ *  callback function, of type "plD_read_pixel_gd", which specifies how a
+ *  single pixel's RGB is read (in the destination context), then
+ *  returns an RGB encoded int with the info for blending.
+\*----------------------------------------------------------------------*/
+
+PLINT plD_read_pixel_gd (PLStream *pls, short x, short y)
+{
+png_Dev *dev=(png_Dev *)pls->dev;
+PLINT colour;
+unsigned char R,G,B;
+
+   colour=gdImageGetTrueColorPixel(dev->im_out, x, y);
+
+   R=gdTrueColorGetRed(colour);
+   G=gdTrueColorGetGreen(colour);
+   B=gdTrueColorGetBlue(colour);
+
+   colour=RGB(R,G,B);
+   return(colour);
+}
+
+
 /*----------------------------------------------------------------------*\
  *  void init_freetype_lv1 (PLStream *pls)
  *
@@ -1278,8 +1398,8 @@ plD_FreeType_init(pls);
 
 FT=(FT_Data *)pls->FT;
 FT->pixel= (plD_pixel_fp)plD_pixel_gd;
-
-
+FT->read_pixel= (plD_read_pixel_fp)plD_read_pixel_gd;
+FT->set_pixel= (plD_set_pixel_fp)plD_set_pixel_gd;
 }
 
 /*----------------------------------------------------------------------*\
@@ -1317,7 +1437,7 @@ FT->ymax=dev->pngy;
 FT->invert_y=1;
 FT->smooth_text=0;
 
-if (FT->want_smooth_text==1)    /* do we want to at least *try* for smoothing ? */
+if ((FT->want_smooth_text==1)&&(FT->BLENDED_ANTIALIASING==0))    /* do we want to at least *try* for smoothing ? */
    {
     FT->ncol0_org=pls->ncol0;                                   /* save a copy of the original size of ncol0 */
     FT->ncol0_xtra=NCOLOURS-(pls->ncol1+pls->ncol0);            /* work out how many free slots we have */
@@ -1344,6 +1464,11 @@ if (FT->want_smooth_text==1)    /* do we want to at least *try* for smoothing ?
     else
       plwarn("Insufficient colour slots available in CMAP0 to do text smoothing.");
    }
+else if ((FT->want_smooth_text==1)&&(FT->BLENDED_ANTIALIASING==1))    /* If we have a truecolour device, we wont even bother trying to change the palette */
+   {
+     FT->smooth_text=1;
+   }
+
 }
 
 #endif
@@ -1359,10 +1484,20 @@ if (FT->want_smooth_text==1)    /* do we want to at least *try* for smoothing ?
 
 void plD_eop_jpeg(PLStream *pls)
 {
-png_Dev *dev=(png_Dev *)pls->dev;
+    png_Dev *dev=(png_Dev *)pls->dev;
+    int im_size=0;
+    void *im_ptr=NULL;
 
     if (pls->family || pls->page == 1) {
-       gdImageJpeg(dev->im_out, pls->OutFile, pls->dev_compression);
+       /* image is written to output file by the driver
+          since if the gd.dll is linked to a different c
+          lib a crash occurs - this fix works also in Linux */
+       /* gdImageJpeg(dev->im_out, pls->OutFile, pls->dev_compression); */
+       im_ptr = gdImageJpegPtr(dev->im_out, &im_size, pls->dev_compression);
+       if( im_ptr ) {
+         fwrite(im_ptr, sizeof(char), im_size, pls->OutFile);
+         gdFree(im_ptr);
+       }
 
        gdImageDestroy(dev->im_out);
        dev->im_out = NULL;
@@ -1381,10 +1516,20 @@ png_Dev *dev=(png_Dev *)pls->dev;
 
 void plD_eop_gif(PLStream *pls)
 {
-png_Dev *dev=(png_Dev *)pls->dev;
+    png_Dev *dev=(png_Dev *)pls->dev;
+    int im_size=0;
+    void *im_ptr=NULL;
 
     if (pls->family || pls->page == 1) {
-       gdImageGif(dev->im_out, pls->OutFile);
+       /* image is written to output file by the driver
+          since if the gd.dll is linked to a different c
+          lib a crash occurs - this fix works also in Linux */
+       /* gdImageGif(dev->im_out, pls->OutFile); */
+       im_ptr = gdImageGifPtr(dev->im_out, &im_size);
+       if( im_ptr ) {
+         fwrite(im_ptr, sizeof(char), im_size, pls->OutFile);
+         gdFree(im_ptr);
+       }
 
        gdImageDestroy(dev->im_out);
        dev->im_out = NULL;
