@@ -1,4 +1,4 @@
-/* $Id: psttf.cc,v 1.8 2006/05/18 11:56:39 rlaboiss Exp $
+/* $Id: psttf.cc 7774 2007-08-02 12:21:22Z andrewross $
 
 	PLplot PostScript device driver using LASi to provide fonts
 	based on original ps.c PostScript driver
@@ -47,6 +47,10 @@
 #include <fstream>
 #include <LASi.h>
 
+/* Define macro to truncate small values to zero - prevents
+ *  * printf printing -0.000 */
+#define TRMFLT(a)     ((fabs(a)<5.0e-4) ? 0.0 : (a))
+
 using namespace LASi;
 using namespace std;
 
@@ -78,11 +82,11 @@ static int hrshsym = 0;
 #define N_Pango_Lookup 5
 
 const char * DefaultFamilyLookup[N_Pango_Lookup] = {
-  "sans",
-  "serif",
-  "monospace",
-  "sans,serif",
-  "sans,serif"
+  "Arial,Bitstream-Vera-Sans,sans",
+  "Times-Roman,Bitstream-Vera-Serif,serif",
+  "Courier,Bitstream-Vera-Sans-Mono,monospace",
+  "Arial,Bitstream-Vera-Sans,sans,serif",
+  "Arial,Bitstream-Vera-Sans,sans,serif"
 };
 
 const char * EnvFamilyLookup[N_Pango_Lookup] = {
@@ -189,6 +193,7 @@ ps_init(PLStream *pls)
     int i;
     char *a;
     PSDev *dev;
+    PostscriptDocument *doc; 
 
     PLFLT pxlx = YPSSIZE/LPAGE_X;
     PLFLT pxly = XPSSIZE/LPAGE_Y;
@@ -215,6 +220,9 @@ ps_init(PLStream *pls)
       delete (PostscriptDocument *) pls->psdoc;
 
     pls->psdoc = new PostscriptDocument();
+    doc = (PostscriptDocument *) (pls->psdoc);
+    doc->osBody() << fixed;
+    doc->osBody().precision(3);
 
 /* Allocate and initialize device-specific data */
 
@@ -599,11 +607,11 @@ plD_tidy_psttf(PLStream *pls)
 /* Write out postscript document to file and close*/
     /* For C++ stream we first need to close the file using 
        the C FILE * handle, then reopen as a ofstream. Yuck! */
-    fclose(pls->OutFile);
     if (! strcmp(pls->FileName,"-")) {
       doc->write(cout,dev->llx,dev->lly,dev->urx,dev->ury);
     }
     else {      
+      fclose(pls->OutFile);
       ofstream out;
       out.open(pls->FileName);
       doc->write(out,dev->llx,dev->lly,dev->urx,dev->ury);
@@ -801,14 +809,9 @@ proc_str (PLStream *pls, EscText *args)
 
   PLFLT scale = 1., up = 0.; /* Font scaling and shifting parameters */
 
-  double lineSpacing, xAdvance, xwid, ymintmp, ymaxtmp, ymin, ymax, xmin, xmax;
+  double lineSpacing, xAdvance, ymintmp, ymaxtmp, ymin, ymax, xmin, xmax;
   PLINT xx[4], yy[4];
 
-  int i=0; /* String index */
-  int k=0;
-
-  short text_len;
-
   FT_Data *FT;
 
    /* unicode only! so test for it. */
@@ -821,7 +824,6 @@ proc_str (PLStream *pls, EscText *args)
 	FontStyle styles[PROC_STR_STRING_LENGTH];
 	FontWeight weights[PROC_STR_STRING_LENGTH];
 	const PLUNICODE *cur_text;
-	const PLUNICODE *cur_text_limit;
 	PLUNICODE fci;
 	unsigned char fontfamily, fontstyle, fontweight;
 	/* translate from unicode into type 1 font index. */
@@ -949,40 +951,107 @@ proc_str (PLStream *pls, EscText *args)
 	         &clipx[2], &clipy[2]);
 	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax,
 	         &clipx[3], &clipy[3]);
-// 	doc->osBody() << " gsave " << clipx[0] << " " << clipy[0] << " " << 
-// 	  clipx[1] << " " << clipy[1] << " " << clipx[2] << " " << 
-// 	  clipy[2] << " " << clipx[3] << " " << clipy[3] << " CL\n";
+ 	doc->osBody() << " gsave " << clipx[0] << " " << clipy[0] << " " << 
+ 	  clipx[1] << " " << clipy[1] << " " << clipx[2] << " " << 
+ 	  clipy[2] << " " << clipx[3] << " " << clipy[3] << " CL\n";
 	
 	/* move to string reference point */
 	doc->osBody() << " " << args->x << " " << args->y << " M\n";
 	
 	/* Save the current position and set the string rotation */
-	doc->osBody() << "gsave " << theta << " R\n";
+	doc->osBody() << "gsave " << TRMFLT(theta) << " R\n";
 
-	doc->osBody() << "[" << tt[0] << " " << tt[2] << " " << tt[1]
-		      << " " << tt[3] << " 0 0] concat\n";
+	doc->osBody() << "[" << TRMFLT(tt[0]) << " " << TRMFLT(tt[2]) << " " << TRMFLT(tt[1])
+		      << " " << TRMFLT(tt[3]) << " 0 0] concat\n";
 	
-	/* Purge escape sequences from string, to find it's 
-	 * length. The string length is computed with the current font, 
-	 * and can thus be wrong if there are font change escape sequences 
-	 * in the string 
-	 */	
-	esc_purge(str, cur_str);
+	xmax = 0;
+	/* Dummy run through the string first to work out the
+	 * length, including any font changes */
+	cur_strp = cur_str;
+	f = 0;
+	do {
+	   strp = str;
+	   
+	   if (*cur_strp == esc) {
+	      cur_strp++;
+	      
+	      if (*cur_strp == esc) { /* <esc><esc> */
+		 *strp++ = *cur_strp++;
+	      }
+	      else if (*cur_strp == 'f') {
+		 cur_strp++;
+		 if (*cur_strp++ != 'f') {
+		    /* escff occurs because of logic above. But any suffix
+		     * other than "f" should never happen. */
+		    plabort("proc_str, internal PLplot logic error;"
+			    "wrong escf escape sequence");
+		    return;
+		 }
+		 font = fonts[f];
+		 style = styles[f];
+		 weight = weights[f];
+		 f++;
+		 continue;
+	      }
+	      else switch (*cur_strp++) {
+		 
+	       case 'd':
+	       case 'D':
+		 if(up>0.) scale *= 1.25;  /* Subscript scaling parameter */
+		 else scale *= 0.8;  /* Subscript scaling parameter */
+		 up -= font_factor * ENLARGE * ft_ht / 2.;
+		 break;
+		 
+	       case 'u':
+	       case 'U':
+		 if(up<0.) scale *= 1.25;  /* Superscript scaling parameter */
+		 else scale *= 0.8;  /* Superscript scaling parameter */
+		 up += font_factor * ENLARGE * ft_ht / 2.;
+		 break;
+		 
+		 /* ignore the next sequences */
+		 
+	       case '+':
+	       case '-':
+	       case 'b':
+	       case 'B':
+		 plwarn("'+', '-', and 'b/B' text escape sequences not processed.");
+		 break;
+	      }
+	   }
+	   
+	   /* copy from current to next token, adding a postscript escape 
+	    * char '\' if necessary 
+	    */
+	   while(*cur_strp && *cur_strp != esc) {
+	      *strp++ = *cur_strp++;
+	   }
+	   *strp = '\0';
+	   
+	   if(fabs(up)<0.001) up = 0.; /* Watch out for small differences */
+	   
+	   /* Set the font size */
+	   doc->setFont(font,style,weight);
+	   doc->setFontSize(font_factor*ENLARGE*ft_ht*scale);
+	   doc->get_dimensions((const char *)str, &lineSpacing, &xAdvance, &ymintmp, &ymaxtmp);
+	   xmax += xAdvance;
 
-	doc->setFont(font,style,weight);
-	doc->setFontSize(font_factor*ENLARGE*ft_ht);
-
-	// Get the approximate length of the string to calculate offset
+	}while(*cur_strp);
+	
+	// Use the length of the string to calculate offset
 	// Also used later for bounding box
-	doc->get_dimensions((const char *)str, &lineSpacing, &xAdvance);
-	xmin = -xAdvance*args->just;
+	xmin = -xmax*args->just;
 	xmax = xmin;
 	ymin = 0;
 	ymax = 0;
 
+	/* Reset parameters */
+	scale = 1.0;
+	up = 0.0;
+
 	/* Move relative to position to account for justification */
-	doc->osBody() << " gsave " << xmin*tt[0] << " " <<
-	  xmin*tt[2] << " rmoveto\n";
+	doc->osBody() << " gsave " << TRMFLT(xmin*tt[0]) << " " <<
+	  TRMFLT(xmin*tt[2]) << " rmoveto\n";
 	
 	/* Parse string for PLplot escape sequences and print everything out */
 	
@@ -1025,8 +1094,8 @@ proc_str (PLStream *pls, EscText *args)
 		 
 	       case 'u':
 	       case 'U':
-		 if(up<0.) scale *= 1.25;  /* Subscript scaling parameter */
-		 else scale *= 0.8;  /* Subscript scaling parameter */
+		 if(up<0.) scale *= 1.25;  /* Superscript scaling parameter */
+		 else scale *= 0.8;  /* Superscript scaling parameter */
 		 up += font_factor * ENLARGE * ft_ht / 2.;
 		 break;
 		 
@@ -1052,8 +1121,8 @@ proc_str (PLStream *pls, EscText *args)
 	   if(fabs(up)<0.001) up = 0.; /* Watch out for small differences */
 	   
 	   /* Set the font size */
-	   doc->setFontSize(font_factor*ENLARGE*ft_ht*scale);
 	   doc->setFont(font,style,weight);
+	   doc->setFontSize(font_factor*ENLARGE*ft_ht*scale);
 	   doc->get_dimensions((const char *)str, &lineSpacing, &xAdvance, &ymintmp, &ymaxtmp);
 	   ymin = MIN(ymintmp+up,ymin);
 	   ymax = MAX(ymaxtmp+up,ymax);
@@ -1062,17 +1131,18 @@ proc_str (PLStream *pls, EscText *args)
 	   /* if up/down escape sequences, save current point and adjust baseline;
 	    * take the shear into account */
 	   if(up!=0.) 
-	     doc->osBody() << "gsave " << up*tt[1] << " " << up*tt[3] << " rmoveto\n";
+	     doc->osBody() << "gsave " << TRMFLT(up*tt[1]) << " " << TRMFLT(up*tt[3]) << " rmoveto\n";
 	   
 	   /* print the string */
 	   doc->osBody() << show((const char *)str);
 	   
 	   /* back to baseline */
 	   if (up!=0.) 
-	     doc->osBody() << "grestore (" << str << ") stringwidth rmoveto\n";
+	     doc->osBody() << "grestore " << TRMFLT(xAdvance*tt[0]) << " " << TRMFLT(xAdvance*tt[2]) << " rmoveto\n";
 	   
 	}while(*cur_strp);
 	
+	doc->osBody() << "grestore\n";
 	doc->osBody() << "grestore\n";
 	doc->osBody() << "grestore\n";
 	
