@@ -1,4 +1,4 @@
-/* $Id: wingcc.c,v 1.13 2005/12/24 14:02:30 arjenmarkus Exp $
+/* $Id: wingcc.c 7335 2007-01-25 09:52:12Z smekal $
 
 	PLplot WIN32 under GCC device driver.
 
@@ -71,6 +71,11 @@ typedef struct {
     PLINT       width;               /* Window width (which can change) */
     PLINT       height;              /* Window Height */
 
+    PLFLT       PRNT_scale;
+    PLINT       PRNT_width;
+    PLINT       PRNT_height;
+
+    char FT_smooth_text;
 /*
  * WIN32 API variables
  */
@@ -83,6 +88,8 @@ typedef struct {
     HPEN          pen;                 /* Windows pen used for drawing */
     HDC           hdc;                 /* Driver Context */
     HDC           hdc2;                /* Driver Context II - used for Blitting */
+    HDC           SCRN_hdc;            /* The screen's context */
+    HDC           PRNT_hdc;            /* used for printing */
     PAINTSTRUCT   ps;                  /* used to paint the client area of a window owned by that application */
     RECT		      rect;                /* defines the coordinates of the upper-left and lower-right corners of a rectangle */
     RECT		      oldrect;             /* used for re-sizing comparisons */
@@ -91,15 +98,17 @@ typedef struct {
     HCURSOR       cursor;              /* Current windows cursor for this window */
     HBITMAP       bitmap;              /* Bitmap of current display; used for fast redraws via blitting */
     HGDIOBJ       oldobject;           /* Used for tracking objects probably not really needed but */
+    HMENU         PopupMenu;
 
     PLINT         draw_mode;
     char          truecolour;          /* Flag to indicate 24 bit mode */
     char          waiting;             /* Flag to indicate drawing is done, and it is waiting; */
                                        /* we only do a windows redraw if plplot is plotting */
     char          enterresize;         /* Used to keep track of reszing messages from windows */
-    char          resize;              /* When "entersize" is set and "reszie" isnt, user maximised the window */
     char          already_erased;      /* Used to track first and only first backgroudn erases */
 
+    struct wingcc_Dev    *push;        /* A copy of the entire structure used when printing */
+                                       /* We push and pop it off a virtual stack */
   } wingcc_Dev;
 
 
@@ -118,6 +127,9 @@ void plD_esc_wingcc        (PLStream *, PLINT, void *);
 
 static void plD_pixel_wingcc (PLStream *pls, short x, short y);
 static void plD_pixelV_wingcc (PLStream *pls, short x, short y);
+static PLINT plD_read_pixel_wingcc (PLStream *pls, short x, short y);
+static void plD_set_pixel_wingcc (PLStream *pls, short x, short y, PLINT colour);
+static void plD_set_pixelV_wingcc (PLStream *pls, short x, short y,PLINT colour);
 static void init_freetype_lv1 (PLStream *pls);
 static void init_freetype_lv2 (PLStream *pls);
 
@@ -137,6 +149,8 @@ static int SetRegValue(char *key_name, char *key_word, char *buffer,int dwType,
 static void Resize ( PLStream *pls );
 static void plD_fill_polygon_wingcc(PLStream *pls);
 static void CopySCRtoBMP(PLStream *pls);
+static void PrintPage ( PLStream *pls );
+static void UpdatePageMetrics ( PLStream *pls, char flag );
 
 #define SetRegStringValue(a,b,c) SetRegValue(a, b, c, REG_SZ, strlen(c)+1 )
 #define SetRegBinaryValue(a,b,c,d) SetRegValue(a, b, (char *)c, REG_BINARY, d )
@@ -149,8 +163,14 @@ static void CopySCRtoBMP(PLStream *pls);
  *  Some debugging macros
 \*--------------------------------------------------------------------------*/
 
-#define Verbose(...) do {if (pls->verbose){fprintf(stderr,__VA_ARGS__);}}while(0)
-#define Debug(...) do {if (pls->debug){fprintf(stderr,__VA_ARGS__);}}while(0)
+#if defined(_MSC_VER)
+  #define Debug(a) do {if (pls->debug){fprintf(stderr,(a));}}while(0)
+#elif defined(__BORLANDC__)
+  #define Debug if (pls->debug) printf
+#else
+  #define Verbose(...) do {if (pls->verbose){fprintf(stderr,__VA_ARGS__);}}while(0)
+  #define Debug(...) do {if (pls->debug){fprintf(stderr,__VA_ARGS__);}}while(0)
+#endif
 
 #define ReportWinError() do { \
 LPVOID lpMsgBuf; \
@@ -176,6 +196,10 @@ dev->cursor = LoadCursor(NULL,IDC_WAIT); \
 SetClassLong(dev->hwnd,GCL_HCURSOR,(long)dev->cursor); \
 SetCursor(dev->cursor);}while(0)
 
+#define PopupPrint              0x08A1
+#define PopupNextPage           0x08A2
+#define PopupQuit               0x08A3
+
 
 void plD_dispatch_init_wingcc( PLDispatchTable *pdt )
 {
@@ -263,14 +287,21 @@ LRESULT CALLBACK PlplotWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lPar
                 if ((dev->waiting==1)&&(dev->already_erased==1))
                   {
                     Debug("Remaking\t");
-                      plRemakePlot(pls);
-                      CopySCRtoBMP(pls);
-                      dev->already_erased++;
-                   }
+
+                    if (dev->ps.fErase)
+                      {
+                        dev->oldcolour = SetBkColor(dev->hdc, RGB(pls->cmap0[0].r,pls->cmap0[0].g,pls->cmap0[0].b));
+                        ExtTextOut(dev->hdc, 0, 0, ETO_OPAQUE, &dev->rect, "", 0, 0);
+                        SetBkColor(dev->hdc, dev->oldcolour);
+                      }
+
+                    plRemakePlot(pls);
+                    CopySCRtoBMP(pls);
+                    dev->already_erased++;
+                  }
                 else if ((dev->waiting==1)&&(dev->already_erased==2))
                   {
                     dev->oldobject = SelectObject(dev->hdc2,dev->bitmap);
-/*                    BitBlt(dev->hdc,0,0,dev->rect.right,dev->rect.bottom,dev->hdc2,0,0,SRCCOPY); */
                     BitBlt(dev->hdc,dev->paintrect.left,dev->paintrect.top,
                            dev->paintrect.right,dev->paintrect.bottom,
                            dev->hdc2,dev->paintrect.left,dev->paintrect.top,SRCCOPY);
@@ -289,10 +320,10 @@ LRESULT CALLBACK PlplotWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lPar
         if (dev)
           {
             Debug("WM_SIZE\t");
+
             if (dev->enterresize==0)
               Resize(pls);
-            else
-              dev->resize=1;
+
           }
         return(0);
       break;
@@ -313,13 +344,12 @@ LRESULT CALLBACK PlplotWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lPar
           	Debug("WM_EXITSIZEMOVE\t");
             Resize(pls);
           	dev->enterresize=0;   /* Reset the variables that track sizing ops */
-          	dev->resize=0;
           }
-
           return(0);
-			break;
+      break;
 
       case WM_ERASEBKGND:
+
       if (dev)
         {
           if (dev->already_erased==0)
@@ -331,26 +361,14 @@ LRESULT CALLBACK PlplotWndProc (HWND hwnd, UINT nMsg, WPARAM wParam, LPARAM lPar
            *    supposidely this executes faster than creating a brush and
            *    filling a rectangle - go figure ?
            */
-/*          GetClientRect(dev->hwnd,&dev->rect); */
+
            dev->oldcolour = SetBkColor(dev->hdc, RGB(pls->cmap0[0].r,pls->cmap0[0].g,pls->cmap0[0].b));
            ExtTextOut(dev->hdc, 0, 0, ETO_OPAQUE, &dev->rect, "", 0, 0);
            SetBkColor(dev->hdc, dev->oldcolour);
 
-          /*
-           *   This is the old "low speed" way of filling the background
-
-
-          dev->bgbrush = CreateSolidBrush(RGB(pls->cmap0[0].r,pls->cmap0[0].g,pls->cmap0[0].b));
-          GetClientRect(dev->hwnd,&dev->rect);
-          SelectObject (dev->hdc, dev->bgbrush);
-          FillRect(dev->hdc, &dev->rect,dev->bgbrush);
-          DeleteObject (dev->bgbrush);
-
-          */
            dev->already_erased=1;
+           return(1);
           }
-
-          return(1);
         }
         return(0);
         break;
@@ -397,7 +415,7 @@ plD_init_wingcc(PLStream *pls)
 
     DrvOpt wingcc_options[] = {
 #ifdef HAVE_FREETYPE
-                              	{"text", DRV_INT, &freetype, "Use driver text (FreeType)"},
+                            	{"text", DRV_INT, &freetype, "Use driver text (FreeType)"},
                               {"smooth", DRV_INT, &smooth_text, "Turn text smoothing on (1) or off (0)"},
                               {"save", DRV_INT, &save_reg, "Save defaults to registary"},
 
@@ -487,7 +505,7 @@ plD_init_wingcc(PLStream *pls)
 	dev->wndclass.cbSize = sizeof(WNDCLASSEX);
 
 	/* All windows of this class redraw when resized. */
-	dev->wndclass.style = CS_HREDRAW | CS_VREDRAW | CS_SAVEBITS | CS_DBLCLKS;
+	dev->wndclass.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_OWNDC | CS_PARENTDC	;
 
 	/* All windows of this class use the WndProc window function. */
 	dev->wndclass.lpfnWndProc = PlplotWndProc;
@@ -503,14 +521,14 @@ plD_init_wingcc(PLStream *pls)
 	/* Color the background white */
 	dev->wndclass.hbrBackground = NULL;
 
-    dev->wndclass.cbWndExtra = sizeof(pls);
+   dev->wndclass.cbWndExtra = sizeof(pls);
 
 
 	/*
 	 * Now register the window class for use.
 	 */
 
-     RegisterClassEx (&dev->wndclass);
+    RegisterClassEx (&dev->wndclass);
 
 
 	/*
@@ -538,7 +556,16 @@ plD_init_wingcc(PLStream *pls)
  */
 
 SetWindowLong(dev->hwnd,GWL_USERDATA,(long)pls);
-dev->hdc = GetDC (dev->hwnd);
+dev->SCRN_hdc = dev->hdc = GetDC (dev->hwnd);
+
+/*
+ *  Setup the popup menu
+ */
+
+dev->PopupMenu=CreatePopupMenu();
+AppendMenu(dev->PopupMenu, MF_STRING, PopupPrint, "Print");
+AppendMenu(dev->PopupMenu, MF_STRING, PopupNextPage, "Next Page");
+AppendMenu(dev->PopupMenu, MF_STRING, PopupQuit, "Quit");
 
 #ifdef HAVE_FREETYPE
 
@@ -569,15 +596,15 @@ plD_state_wingcc(pls, PLSTATE_COLOR0);
     */
 
 
-     if (pls->xdpi<=0) /* Get DPI from windows */
-     {
-         plspage(GetDeviceCaps(dev->hdc,HORZRES)/GetDeviceCaps(dev->hdc,HORZSIZE)*25.4,
-         GetDeviceCaps(dev->hdc,VERTRES)/GetDeviceCaps(dev->hdc,VERTSIZE)*25.4, 0, 0, 0, 0);
-     }
-     else
-     {
+    if (pls->xdpi<=0) /* Get DPI from windows */
+      {
+        plspage(GetDeviceCaps(dev->hdc,HORZRES)/GetDeviceCaps(dev->hdc,HORZSIZE)*25.4,
+        GetDeviceCaps(dev->hdc,VERTRES)/GetDeviceCaps(dev->hdc,VERTSIZE)*25.4, 0, 0, 0, 0);
+      }
+    else
+      {
         pls->ydpi=pls->xdpi;        /* Set X and Y dpi's to the same value */
-     }
+      }
 
 
 /*
@@ -588,7 +615,6 @@ plD_state_wingcc(pls, PLSTATE_COLOR0);
  */
 
   	 GetClientRect(dev->hwnd,&dev->rect);
-/*    GetClipBox(dev->hdc,&dev->rect); */
     dev->width=dev->rect.right;
     dev->height=dev->rect.bottom;
 
@@ -604,7 +630,6 @@ plD_state_wingcc(pls, PLSTATE_COLOR0);
      Debug("Scale = %f (FLT)\n",dev->scale);
 
      plP_setpxl(dev->scale*pls->xdpi/25.4,dev->scale*pls->ydpi/25.4);
-
      plP_setphy(0, dev->scale*dev->width, 0, dev->scale*dev->height);
 
 #ifdef HAVE_FREETYPE
@@ -689,6 +714,7 @@ plD_fill_polygon_wingcc(PLStream *pls)
   wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
   int i;
   POINT *points=NULL;
+  HPEN hpen, hpenOld;
 
   if (pls->dev_npts > 0)
     {
@@ -704,10 +730,14 @@ plD_fill_polygon_wingcc(PLStream *pls)
         }
 
       dev->fillbrush = CreateSolidBrush(dev->colour);
+      hpen = CreatePen(PS_SOLID, 1, dev->colour);
       dev->oldobject = SelectObject (dev->hdc, dev->fillbrush);
+      hpenOld = SelectObject(dev->hdc, hpen);
       Polygon(dev->hdc,points,pls->dev_npts);
       SelectObject (dev->hdc, dev->oldobject);
       DeleteObject (dev->fillbrush);
+      SelectObject (dev->hdc, hpenOld);
+      DeleteObject (hpen);
       GlobalFree(points);
     }
 }
@@ -722,7 +752,6 @@ plD_fill_polygon_wingcc(PLStream *pls)
 static void CopySCRtoBMP(PLStream *pls)
 {
   wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
-  RECT rect;
 
   /*
    *   Clean up the old bitmap and DC
@@ -732,11 +761,10 @@ static void CopySCRtoBMP(PLStream *pls)
   if (dev->bitmap!=NULL) DeleteObject(dev->bitmap);
 
   dev->hdc2=CreateCompatibleDC(dev->hdc);
-/*  GetClipBox(dev->hdc,&rect);
-  GetClientRect(dev->hwnd,&dev->rect); */
+  GetClientRect(dev->hwnd,&dev->rect);
   dev->bitmap=CreateCompatibleBitmap(dev->hdc,dev->rect.right,dev->rect.bottom);
   dev->oldobject=SelectObject(dev->hdc2,dev->bitmap);
-  BitBlt(dev->hdc2,0,0,rect.right,rect.bottom,dev->hdc,0,0,SRCCOPY);
+  BitBlt(dev->hdc2,0,0,dev->rect.right,dev->rect.bottom,dev->hdc,0,0,SRCCOPY);
   SelectObject(dev->hdc2,dev->oldobject);
 }
 
@@ -762,6 +790,12 @@ plD_eop_wingcc(PLStream *pls)
                 switch((int)dev->msg.message)
                   {
 
+                    case WM_CONTEXTMENU:
+                    case WM_RBUTTONDOWN:
+                       TrackPopupMenu(dev->PopupMenu,TPM_CENTERALIGN | TPM_RIGHTBUTTON,LOWORD(dev->msg.lParam),
+                          HIWORD(dev->msg.lParam),0,dev->hwnd,NULL);
+                    break;
+
                     case WM_CHAR:
                     if (((TCHAR)(dev->msg.wParam)== 32)||
                         ((TCHAR)(dev->msg.wParam)== 13))
@@ -782,6 +816,25 @@ plD_eop_wingcc(PLStream *pls)
                       dev->waiting=0;
                     break;
 
+                    case WM_COMMAND:
+                      switch(LOWORD(dev->msg.wParam))
+                            {
+                              case PopupPrint:
+                                Debug("PopupPrint");
+                                PrintPage(pls);
+                              break;
+                              case PopupNextPage:
+                                Debug("PopupNextPage");
+                                dev->waiting=0;
+                              break;
+                              case PopupQuit:
+                                Debug("PopupQuit");
+                                dev->waiting=0;
+                                PostQuitMessage(0);
+                              break;
+                            }
+                     break;
+
                     default:
                       DispatchMessage (&dev->msg);
                     break;
@@ -818,6 +871,7 @@ plD_bop_wingcc(PLStream *pls)
 void
 plD_tidy_wingcc(PLStream *pls)
 {
+  wingcc_Dev *dev = NULL;
 
 #ifdef HAVE_FREETYPE
   if (pls->dev_text)
@@ -827,9 +881,24 @@ plD_tidy_wingcc(PLStream *pls)
       plD_FreeType_Destroy(pls);
     }
 #endif
+  Debug("plD_tidy_wingcc");
 
   if (pls->dev!=NULL)
-    free(pls->dev);
+    {
+      dev = (wingcc_Dev *)pls->dev;
+
+      DeleteMenu(dev->PopupMenu, PopupPrint, 0);
+      DeleteMenu(dev->PopupMenu, PopupNextPage, 0);
+      DeleteMenu(dev->PopupMenu, PopupQuit, 0);
+      DestroyMenu(dev->PopupMenu);
+
+      if (dev->hdc2!=NULL) DeleteDC(dev->hdc2);
+      if (dev->hdc!=NULL) ReleaseDC(dev->hwnd,dev->hdc);
+      if (dev->bitmap!=NULL) DeleteObject(dev->bitmap);
+
+      free_mem(pls->dev);
+    }
+
 }
 
 
@@ -928,9 +997,9 @@ static void Resize( PLStream *pls )
  	if (dev->waiting==1)     /* Only resize the window IF plplot has finished with it */
   	{
   	   memcpy(&dev->oldrect,&dev->rect,sizeof(RECT));
-/*   	GetClipBox(dev->hdc,&dev->rect); */                 /* Find out how big the region is NOW */
       GetClientRect(dev->hwnd,&dev->rect);
       Debug("[%d %d]",dev->rect.right,dev->rect.bottom);
+
    	if ((dev->rect.right>0)&&(dev->rect.bottom>0))    /* Check to make sure it isn't just minimised (i.e. zero size) */
         {
           if (memcmp(&dev->rect,&dev->oldrect,sizeof(RECT))!=0)   /* See if the window's changed size or not */
@@ -955,9 +1024,7 @@ static void Resize( PLStream *pls )
                 }
 #endif
            }
-          RedrawWindow(dev->hwnd,NULL,NULL,RDW_INVALIDATE|RDW_ERASE|RDW_ERASENOW);
-          SendMessage(dev->hwnd,WM_PAINT,0,0);
-          dev->already_erased=2;
+          RedrawWindow(dev->hwnd,NULL,NULL,RDW_ERASE|RDW_INVALIDATE|RDW_ERASENOW);
         }
        else
          {
@@ -1038,20 +1105,54 @@ static int GetRegValue(char *key_name, char *key_word, char *buffer, int size)
  *  pixel is set in the current colour.
 \*----------------------------------------------------------------------*/
 
-void plD_pixel_wingcc (PLStream *pls, short x, short y)
+static void plD_pixel_wingcc (PLStream *pls, short x, short y)
 {
   wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
 
    SetPixel(dev->hdc, x, y,dev->colour);
-
 }
 
-void plD_pixelV_wingcc (PLStream *pls, short x, short y)
+static void plD_pixelV_wingcc (PLStream *pls, short x, short y)
 {
   wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
 
    SetPixelV(dev->hdc, x, y,dev->colour);
+}
 
+/*----------------------------------------------------------------------*\
+ *  void plD_set_pixelV_wingcc (PLStream *pls, short x, short y,PLINT colour)
+ *
+ *  callback function, of type "plD_set_pixel_fp", which specifies how a 
+ *  single pixel is set in the s[ecified colour. This colour
+ *  by-passes plplot's internal table, and directly 'hits the hardware'.
+\*----------------------------------------------------------------------*/
+
+static void plD_set_pixel_wingcc (PLStream *pls, short x, short y, PLINT colour)
+{
+  wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
+
+   SetPixel(dev->hdc, x, y,colour);
+}
+
+static void plD_set_pixelV_wingcc (PLStream *pls, short x, short y, PLINT colour)
+{
+  wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
+
+   SetPixelV(dev->hdc, x, y,colour);
+}
+
+
+/*--------------------------------------------------------------------------*\
+ *  void plD_read_pixel_wingcc (PLStream *pls, short x, short y)
+ *
+ *  callback function, of type "plD_pixel_fp", which specifies how a single
+ *  pixel is read.
+\*--------------------------------------------------------------------------*/
+static PLINT plD_read_pixel_wingcc (PLStream *pls, short x, short y)
+{
+  wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
+
+   return(GetPixel(dev->hdc, x, y));
 }
 
 
@@ -1075,6 +1176,7 @@ plD_FreeType_init(pls);
 FT=(FT_Data *)pls->FT;
 
 
+
 /*
  *  Work out if our device support "fast" pixel setting
  *  and if so, use that instead of "slow" pixel setting
@@ -1087,6 +1189,21 @@ if (x & RC_BITBLT)
 else
   FT->pixel= (plD_pixel_fp)plD_pixel_wingcc;
 
+/*
+ *  See if we have a 24 bit device (or better), in which case
+ *  we can use the better antialaaising.
+ */
+
+if (GetDeviceCaps(dev->hdc,BITSPIXEL)>24)
+{
+  FT->BLENDED_ANTIALIASING=1;
+  FT->read_pixel= (plD_read_pixel_fp)plD_read_pixel_wingcc;
+
+  if (x & RC_BITBLT)
+    FT->set_pixel= (plD_set_pixel_fp)plD_set_pixelV_wingcc;
+  else
+    FT->set_pixel= (plD_set_pixel_fp)plD_set_pixel_wingcc;
+}
 
 }
 
@@ -1095,7 +1212,7 @@ else
  *
  *  "Level 2" initialisation of the freetype library.
  *  "Level 2" fills in a few setting that aren't public until after the
- *  graphics sub-syetm has been initialised.
+ *  graphics sub-system has been initialised.
  *  The "level 2" initialisation fills in a few things that are defined
  *  later in the initialisation process for the GD driver.
  *
@@ -1124,7 +1241,7 @@ FT->scale=dev->scale;
 FT->ymax=dev->height;
 FT->invert_y=1;
 
-if (FT->want_smooth_text==1)    /* do we want to at least *try* for smoothing ? */
+if ((FT->want_smooth_text==1)&&(FT->BLENDED_ANTIALIASING==0))    /* do we want to at least *try* for smoothing ? */
    {
     FT->ncol0_org=pls->ncol0;                                   /* save a copy of the original size of ncol0 */
     FT->ncol0_xtra=16777216-(pls->ncol1+pls->ncol0);            /* work out how many free slots we have */
@@ -1144,10 +1261,174 @@ if (FT->want_smooth_text==1)    /* do we want to at least *try* for smoothing ?
           pls->level = level_save;
          }
         FT->smooth_text=1;      /* Yippee ! We had success setting up the extended cmap0 */
+    }
+else if ((FT->want_smooth_text==1)&&(FT->BLENDED_ANTIALIASING==1))    /* If we have a truecolour device, we wont even bother trying to change the palette */
+   {
+     FT->smooth_text=1;
+   }
+}
+#endif
+
+/*--------------------------------------------------------------------------*\
+ *  static void UpdatePageMetrics ( PLStream *pls, char flag )
+ *
+ *      UpdatePageMetrics is a simple function which simply gets new vales for
+ *      a changed DC, be it swapping from printer to screen or vice-versa.
+ *      The flag variable is used to tell the function if it is updating
+ *      from the printer (1) or screen (0).
+\*--------------------------------------------------------------------------*/
+
+static void UpdatePageMetrics ( PLStream *pls, char flag )
+{
+  wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
+  #ifdef HAVE_FREETYPE
+  FT_Data *FT=(FT_Data *)pls->FT;
+  #endif
+
+  if (flag==1)
+    {
+      dev->width=GetDeviceCaps(dev->hdc,HORZRES);   /* Get the page size from the printer */
+      dev->height=GetDeviceCaps(dev->hdc,VERTRES);
+    }
+  else
+    {
+      GetClientRect(dev->hwnd,&dev->rect);
+      dev->width=dev->rect.right;
+      dev->height=dev->rect.bottom;
+    }
+
+  if (dev->width>dev->height)           /* Work out the scaling factor for the  */
+    {                                   /* "virtual" (oversized) page           */
+      dev->scale=(PLFLT)PIXELS_X/dev->width;
+    }
+  else
+    {
+      dev->scale=(PLFLT)PIXELS_Y/dev->height;
+    }
+
+  #ifdef HAVE_FREETYPE
+    if (FT)           /* If we are using freetype, then set it up next */
+      {
+        FT->scale=dev->scale;
+        FT->ymax=dev->height;
+        if (GetDeviceCaps(dev->hdc,RASTERCAPS) & RC_BITBLT)
+          FT->pixel= (plD_pixel_fp)plD_pixelV_wingcc;
+        else
+          FT->pixel= (plD_pixel_fp)plD_pixel_wingcc;
       }
+  #endif
+
+  pls->xdpi=GetDeviceCaps(dev->hdc,HORZRES)/GetDeviceCaps(dev->hdc,HORZSIZE)*25.4;
+  pls->ydpi=GetDeviceCaps(dev->hdc,VERTRES)/GetDeviceCaps(dev->hdc,VERTSIZE)*25.4;
+  plP_setpxl(dev->scale*pls->xdpi/25.4,dev->scale*pls->ydpi/25.4);
+  plP_setphy(0, dev->scale*dev->width, 0, dev->scale*dev->height);
+
 }
 
-#endif
+/*--------------------------------------------------------------------------*\
+ *  static void PrintPage ( PLStream *pls )
+ *
+ *     Function brings up a standard printer dialog and, after the user
+ *     has selected a printer, replots the current page to the windows
+ *     printer.
+\*--------------------------------------------------------------------------*/
+
+static void PrintPage ( PLStream *pls )
+{
+  wingcc_Dev *dev=(wingcc_Dev *)pls->dev;
+  #ifdef HAVE_FREETYPE
+  FT_Data *FT=(FT_Data *)pls->FT;
+  #endif
+  PRINTDLG  Printer;
+  DOCINFO docinfo;
+  char *PrinterName;
+  BOOL ret;
+
+  /*
+   *    Reset the docinfo structure to 0 and set it's fields up
+   *    This structure is used to supply a name to the print queue
+   */
+
+  ZeroMemory(&docinfo,sizeof(docinfo));
+  docinfo.cbSize=sizeof(docinfo);
+  docinfo.lpszDocName="Plplot Page";
+
+  /*
+   *   Reset out printer structure to zero and initialise it
+   */
+
+  ZeroMemory(&Printer,sizeof(PRINTDLG));
+  Printer.lStructSize=sizeof(PRINTDLG);
+  Printer.hwndOwner=dev->hwnd;
+  Printer.Flags=PD_NOPAGENUMS|PD_NOSELECTION|PD_RETURNDC;
+  Printer.nCopies=1;
+
+  /*
+   *   Call the printer dialog function.
+   *   If the user has clicked on "Print", then we will continue
+   *   processing and print out the page.
+   */
+
+  if (PrintDlg(&Printer)!=0)
+    {
+
+      /*
+       *  Before doing anything, we will take some backup copies
+       *  of the existing values for page size and the like, because
+       *  all we are going to do is a quick and dirty modification
+       *  of plplot's internals to match the new page size and hope
+       *  it all works out ok. After that, we will manip the values,
+       *  and when all is done, restore them.
+       */
+
+      if ((dev->push=GlobalAlloc(GMEM_ZEROINIT,sizeof(wingcc_Dev)))!=NULL)
+        {
+          BusyCursor();
+          memcpy(dev->push,dev,sizeof(wingcc_Dev));
+
+          dev->hdc=dev->PRNT_hdc=Printer.hDC;      /* Copy the printer HDC */
+
+          UpdatePageMetrics ( pls, 1 );
+
+          #ifdef HAVE_FREETYPE
+            if (FT)           /* If we are using freetype, then set it up next */
+              {
+                dev->FT_smooth_text=FT->smooth_text; /* When printing, we don't want smoothing */
+                FT->smooth_text=0;
+              }
+          #endif
+
+          /*
+           *   Now the stuff that actually does the printing !!
+           */
+
+          StartDoc(dev->hdc,&docinfo);
+          plRemakePlot(pls);
+          EndDoc(dev->hdc);
+
+          /*
+           *  Now to undo everything back to what it was for the screen
+           */
+
+          dev->hdc=dev->SCRN_hdc;      /* Reset the screen HDC to the default */
+          UpdatePageMetrics ( pls, 0 );
+
+          #ifdef HAVE_FREETYPE
+            if (FT)           /* If we are using freetype, then set it up next */
+              {
+                FT->smooth_text=dev->FT_smooth_text;
+              }
+          #endif
+          memcpy(dev,dev->push,sizeof(wingcc_Dev));   /* POP our "stack" now to restore the values */
+
+          GlobalFree(dev->push);
+          NormalCursor();
+          RedrawWindow(dev->hwnd,NULL,NULL,RDW_ERASE|RDW_INVALIDATE|RDW_ERASENOW);
+        }
+  }
+}
+
+
 
 #else
 int
