@@ -1,4 +1,4 @@
-/* $Id: xwin.c,v 1.127 2005/04/27 06:43:48 rlaboiss Exp $
+/* $Id: xwin.c 7044 2006-09-05 05:10:13Z mlebrun $
 
 	PLplot X-windows device driver.
 
@@ -2226,6 +2226,9 @@ ResizeCmd(PLStream *pls, PLDisplay *pldis)
 	CreatePixmap(pls);
     }
 
+/* This allows an external agent to take over the redraw */
+    if (pls->ext_resize_draw) return;
+
 /* Initialize & redraw (to pixmap, if available). */
 
     plD_bop_xw(pls);
