@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x21.cc,v 1.14 2006/05/27 21:50:26 hbabcock Exp $
+// $Id: x21.cc 7038 2006-08-31 17:34:18Z airwin $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -33,6 +33,11 @@
   #define isnan(x) ((x) != (x))
 #endif
 
+#if defined(WIN32) && (defined(_MSC_VER) || defined(__BORLANDC__))
+#undef isnan
+#define isnan _isnan
+#endif
+
 // Need for some Mac OSX systems with broken <cmath> header
 #ifdef BROKEN_ISNAN_CXX
 extern "C" int isnan (double);
@@ -42,6 +47,11 @@ extern "C" int isnan (double);
 using namespace std;
 #endif
 
+/* Function drand48() does not exist on all platforms
+   and is deprecated on others. Use rand() instead
+*/
+#define drand48 (1.0/RAND_MAX)*(PLFLT)rand
+
 class x21 {
 
 public:
@@ -344,7 +354,7 @@ void x21::cmap1_init() {
   s[1] = 0.8;
 
   pls->scmap1n(256);
-  pls->scmap1l(false, 2, i, h, l, s, NULL);
+  pls->scmap1l(false, 2, i, h, l, s);
 }
 
 void x21::create_grid(PLFLT **xi, PLINT px, PLFLT **yi, PLINT py) {
