@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x06.java,v 1.13 2005/07/03 12:43:39 andrewross Exp $
+// $Id: x06.java 6815 2006-07-19 07:13:56Z andrewross $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -31,14 +31,13 @@ package plplot.examples;
 
 import plplot.core.*;
 
-import java.lang.Math;
 import java.text.*;
 
 class x06 {
 
     public static void main( String[] args ) 
     {
-        x06 x = new x06( args );
+        new x06( args );
     }
 
     public x06( String[] args )
