@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x08.java,v 1.29 2005/12/08 08:39:23 andrewross Exp $
+// $Id: x08.java 6815 2006-07-19 07:13:56Z andrewross $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -102,7 +102,7 @@ class x08 {
 
     public static void main( String[] args )
     {
-        x08 x = new x08( args );
+        new x08( args );
     }
 
     public x08( String[] args )
