@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x09.java,v 1.23 2005/07/03 12:43:39 andrewross Exp $
+// $Id: x09.java 6815 2006-07-19 07:13:56Z andrewross $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -64,7 +64,7 @@ class x09 {
 
     public static void main( String[] args ) 
     {
-        x09 x = new x09( args );
+        new x09( args );
     }
 
     public x09( String[] args )
