@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x13.java,v 1.13 2005/07/03 12:43:39 andrewross Exp $
+// $Id: x13.java 6815 2006-07-19 07:13:56Z andrewross $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -32,7 +32,6 @@ package plplot.examples;
 import plplot.core.*;
 
 import java.lang.Math;
-import java.text.*;
 
 class x13 {
 
@@ -41,7 +40,7 @@ class x13 {
 
     public static void main( String[] args ) 
     {
-        x13 x = new x13( args );
+        new x13( args );
     }
 
     public x13( String[] args )
