@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x15.java,v 1.16 2005/12/08 08:39:23 andrewross Exp $
+// $Id: x15.java 6815 2006-07-19 07:13:56Z andrewross $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -31,8 +31,6 @@ package plplot.examples;
 
 import plplot.core.*;
 
-import java.lang.Math;
-
 class x15 {
 
    static final int XPTS = 35;
@@ -65,7 +63,7 @@ class x15 {
 
     public static void main( String[] args ) 
     {
-        x15 x = new x15( args );
+        new x15( args );
     }
 
     public x15( String[] args )
