@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x22.java,v 1.10 2005/12/08 08:39:23 andrewross Exp $
+// $Id: x22.java 6815 2006-07-19 07:13:56Z andrewross $
 //    Simple vector plot example
 //---------------------------------------------------------------------------//
 //
@@ -288,7 +288,7 @@ class x22 {
     }
 
     public static void main( String[] args ) {
-	x22 x = new x22( args );
+	new x22( args );
 
     }
 }
