@@ -9,10 +9,10 @@ animation.py - Demonstrates the use of the plplot canvas widget with gtk.
   Department of Physics and Atmospheric Science,
   Dalhousie University, Halifax, Nova Scotia, Canada, B3H 3J5
 
-  $Author: tomduck $
-  $Revision: 1.8 $
-  $Date: 2005/05/16 17:07:00 $
-  $Name: cvs-tarball_2006-05-29-23-03-54 $
+  $Author: andrewross $
+  $Revision: 7770 $
+  $Date: 2007-08-01 11:32:29 -0400 (Wed, 01 Aug 2007) $
+  $Name$
 
 
 NOTICE
@@ -41,10 +41,9 @@ DESCRIPTION
   to the two graphs is handled in two different threads.
 """
 import sys,threading
-import Numeric
 import plplot_python_start
+import plplot_py_demos
 import plplotcanvas
-import plplot
 import gtk
 
 
@@ -68,7 +67,7 @@ canvas0=None
 canvas1=None
 
 # Create the x array
-x = Numeric.arange(NPTS)
+x = ${PYTHON_NUMERIC_NAME}.arange(NPTS)
 
 # Lock on the gtkstate so that we don't try to plot after gtk_main_quit
 GTKSTATE_CONTINUE=True
@@ -109,7 +108,7 @@ def plot(canvas,offset,title):
     Nstream = canvas.get_stream_number()
 
     # Generate the sinusoid
-    y = Numeric.sin(2.*3.14*(x+offset*(Nstream+1))/PERIOD/(Nstream+1))
+    y = ${PYTHON_NUMERIC_NAME}.sin(2.*3.14*(x+offset*(Nstream+1))/PERIOD/(Nstream+1))
     
     # Draw the line
     canvas.line(x, y)
