@@ -33,16 +33,23 @@ plparseopts(sys.argv, PL_PARSE_FULL)
 # Initialize plplot
 plinit()
 
-# xw14.py must be standalone so cannot be run from pythondemos.py
-# xw17.py must be standalone and interactive so cannot be run from pythondemos.py
-
+# Skip 14 (must be standalone so cannot be run from pythondemos.py),
+# 17 (must be standalone and interactive so cannot be run from pythondemos.py),
+# 19 (placeholder without real implementation),
+# 20 (no implementation)
+# 21 (no implementation), and 
+# 24 (requires special fonts installed to produce good-looking result).
+# 25 (no implementation)
 failed = []
-for i in range(1, 14) + [15, 16, 18, 19, 22]:
+for i in range(1, 14) + [15, 16, 18, 22, 23, 26]:
     script = 'xw' + '%02d' % i
     try:
 	__import__(script, globals(), locals(), [])
     except:
 	failed.append(script)
+    # Reset color here rather than in individual examples to ensure results
+    # for individual tests are identical to C versions
+    plcol0(1)
 
 # Terminate plplot
 plend()
