@@ -1,5 +1,4 @@
-from Numeric import *
-from plplot import *
+from plplot_py_demos import *
 
 XPTS = 35		# Data points in x
 YPTS = 46		# Data points in y
@@ -32,14 +31,10 @@ def restore_cmap1():
     l = array((0.5, midpt, vertex, vertex, midpt, 0.5))
     # Saturation is complete for default
     s = array((1., 1., 1., 1., 1., 1.))
-    # Integer flag array is zero (no interpolation along far-side of colour
-    # figure.)
-    # Drop array, see plplot-devel discussion on 2004-10-27
-    rev = array((0, 0, 0, 0, 0, 0))
     # Default number of cmap1 colours
     plscmap1n(128)
     # Interpolate between control points to set up default cmap1.
-    plscmap1l(0, i, h, l, s, rev)
+    plscmap1l(0, i, h, l, s)
 
 # Routine for defining a specific color map 1 in HLS space.
 # if gray is true, use basic grayscale variation from half-dark to light.
@@ -61,14 +56,10 @@ def cmap1_init(gray):
 	l = array((0.6, 0.6))
 	s = array((0.8, 0.8))
 
-    # Integer flag array is zero (no interpolation along far-side of colour
-    # figure.)
-    # Drop array, see plplot-devel discussion on 2004-10-27
-    rev = array((0, 0))
     # number of cmap1 colours is 256 in this case.
     plscmap1n(256)
     # Interpolate between control points to set up cmap1.
-    plscmap1l(0, i, h, l, s, rev)
+    plscmap1l(0, i, h, l, s)
 # main
 #
 # Does a series of mesh plots for a given data set, with different
@@ -76,8 +67,8 @@ def cmap1_init(gray):
 
 def main():
 
-    x = 3.*(arrayrange(XPTS) - (XPTS / 2)) / float(XPTS / 2)
-    y = 3.*(arrayrange(YPTS) - (YPTS / 2)) / float(YPTS / 2)
+    x = 3.*(arange(XPTS) - (XPTS / 2)) / float(XPTS / 2)
+    y = 3.*(arange(YPTS) - (YPTS / 2)) / float(YPTS / 2)
     x.shape = (-1,1)
     z = 3. * (1.-x)*(1.-x) * exp(-(x*x) - (y+1.)*(y+1.)) - \
         10. * (x/5. - pow(x,3.) - pow(y,5.)) * exp(-x*x-y*y) - \
@@ -127,6 +118,6 @@ def main():
 	    plmtex("t", 1.0, 0.5, 0.5, title[k])
 
     # Restore defaults
-    plcol0(1)
+    #plcol0(1)
     restore_cmap1()
 main()
