@@ -1,5 +1,4 @@
-from Numeric import *
-from plplot import *
+from plplot_py_demos import *
 
 text = ["Maurice",
 	"Geoffrey",
@@ -29,7 +28,7 @@ def main():
 	if i == 4:
 	    theta1 = 500
 
-	theta = arrayrange(theta0,theta1+1)
+	theta = arange(theta0,theta1+1)
 	x = concatenate((array([5.]), (5. + 3.*cos((2.*pi/500.)*theta))))
 	y = concatenate((array([5.]), (5. + 3.*sin((2.*pi/500.)*theta))))
 
@@ -56,6 +55,6 @@ def main():
 
     # Restore defaults
     plfont(1)
-    plcol0(1)
+    #plcol0(1)
 	
 main()
