@@ -22,7 +22,7 @@
 from plplot_python_start import *
 
 import sys
-from plplot import *
+from plplot_py_demos import *
 
 # Parse and process command line arguments
 plparseopts(sys.argv, PL_PARSE_FULL)
@@ -30,7 +30,7 @@ plparseopts(sys.argv, PL_PARSE_FULL)
 # Initialize plplot
 plinit()
 
-from Numeric import *
+from ${PYTHON_NUMERIC_NAME} import *
 from random import random
 from time import sleep
 
@@ -85,7 +85,7 @@ def main():
 # Register our error variables with PLplot 
 # From here on, we're handling all errors here 
 
-    #plsError(&errcode, errmsg)
+    #plsError(&pl_errcode, errmsg)
 
     id1 = plstripc("bcnst", "bcnstv",
                    tmin, tmax, tjump, ymin, ymax,
