@@ -1,5 +1,5 @@
-/* $Id: font11.c,v 1.1 1992/05/20 21:33:32 furnish Exp $
-   $Log: font11.c,v $
+/* $Id: font11.c 2 1992-05-20 21:36:02Z furnish $
+   $Log$
    Revision 1.1  1992/05/20 21:33:32  furnish
    Initial checkin of the whole PLPLOT project.
 
