@@ -1,4 +1,4 @@
-/* $Id: plcore.h,v 1.77 2006/04/17 11:54:12 andrewross Exp $
+/* $Id: plcore.h 7708 2007-06-03 22:08:58Z hbabcock $
 
 	Contains declarations for core plplot data structures.  This file
 	should be included only by plcore.c.
@@ -86,7 +86,7 @@ static PLStream *pls[PL_NSTREAMS] = {&pls0};	/* Array of stream pointers */
 
 /* Current stream pointer.  Global, for easier access to state info */
 
-PLStream *plsc = &pls0;
+PLDLLIMPEXP_DATA(PLStream) *plsc = &pls0;
 
 /* Only now can we include this */
 
@@ -258,10 +258,35 @@ static PLDispatchInit static_device_initializers[] = {
 #if defined(PLD_wxwidgets) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_wxwidgets,
 #endif
+#if defined(PLD_svg) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_svg,
+#endif
+#if defined(PLD_pdf) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_pdf,
+#endif
 #if defined(PLD_psttf) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_psttfm,
     plD_dispatch_init_psttfc,
 #endif
+#if defined(PLD_xcairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_xcairo,
+#endif
+#if defined(PLD_pdfcairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_pdfcairo,
+#endif
+#if defined(PLD_pscairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_pscairo,
+#endif
+#if defined(PLD_svgcairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_svgcairo,
+#endif
+#if defined(PLD_pngcairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_pngcairo,
+#endif
+#if defined(PLD_memcairo) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_memcairo,
+#endif
+
     NULL
 };
 
