@@ -17,6 +17,9 @@
 #if !defined(_CSA_H)
 #define _CSA_H
 
+/* include header file for dll definitions */
+#include "csadll.h"
+
 #if !defined(_POINT_STRUCT)
 #define _POINT_STRUCT
 typedef struct {
@@ -32,16 +35,26 @@ extern char* csa_version;
 struct csa;
 typedef struct csa csa;
 
+CSADLLIMPEXP
 csa* csa_create();
+CSADLLIMPEXP
 void csa_destroy(csa* a);
+CSADLLIMPEXP
 void csa_addpoints(csa* a, int n, point points[]);
+CSADLLIMPEXP
 void csa_calculatespline(csa* a);
+CSADLLIMPEXP
 void csa_approximate_point(csa* a, point* p);
+CSADLLIMPEXP
 void csa_approximate_points(csa* a, int n, point* points);
 
+CSADLLIMPEXP
 void csa_setnpmin(csa* a, int npmin);
+CSADLLIMPEXP
 void csa_setnpmax(csa* a, int npmax);
+CSADLLIMPEXP
 void csa_setk(csa* a, int k);
+CSADLLIMPEXP
 void csa_setnpps(csa* a, double npps);
 
 #endif
