@@ -19,11 +19,18 @@
 #if !defined(_NAN_H)
 #define _NAN_H
 
-#if defined(__GNUC__) && !defined(__ICC)
+#if (defined(__GNUC__) && !defined(__ICC)) || defined(__BORLANDC__)
 
 static const double NaN = 0.0 / 0.0;
 
-#elif defined(_WIN32)
+#ifdef __BORLANDC__
+  #define isnan _isnan
+  #define copysign _copysign
+#endif
+
+#elif defined(WIN32)
+
+#if !defined(_MSC_VER)
 
 static unsigned _int64 lNaN = ((unsigned _int64) 1 << 63) - 1;
 
@@ -31,6 +38,25 @@ static unsigned _int64 lNaN = ((unsigned _int64) 1 << 63) - 1;
 
 #else
 
+/* MSVC/C++ 6.0 reports a version 1200,
+   MSVC/C++ 2003 reports a version 1300
+*/
+#include <float.h>
+#include <ymath.h>
+#undef NaN
+#if _MSC_VER < 1300
+#define NaN _Nan._D
+#else
+#define NaN _Nan._Double
+#endif
+
+#define isnan _isnan
+#define copysign _copysign
+
+#endif
+
+#else
+
 static const long long lNaN = ((unsigned long long) 1 << 63) - 1;
 
 #define NaN (*(double*)&lNaN)
