@@ -25,60 +25,57 @@
 build=${build:-true}
 
 if test "$build" = true; then
-    make clean
-    make configure
-
-    ./configure --disable-static --enable-dyndrivers --prefix=/tmp/plplot \
-    --disable-cxx --disable-python --disable-java --disable-itcl \
-    --disable-octave --enable-f77 --enable-psttf --enable-psttfc
-
+#    make clean
+#    make configure
+#
+#    ./configure --disable-static --enable-dyndrivers --prefix=/tmp/plplot \
+#    --disable-cxx --disable-python --disable-java --disable-itcl \
+#    --disable-octave --enable-f77 --enable-psttf --enable-psttfc
+#
+    rm -rf htdocsgen/build_dir htdocsgen/install
+    mkdir -p htdocsgen/build_dir htdocsgen/install
+    cd htdocsgen/build_dir
+    # Specifically enable f77 so that examples will be generated.
+    cmake -DCMAKE_INSTALL_PREFIX=`pwd`/../install \
+    -DDEFAULT_NO_BINDINGS=ON -DENABLE_f77=ON \
+    -DDEFAULT_NO_DEVICES=ON -DPLD_pngcairo=ON \
+    ../../
     make
     make install
+    cd ../..
 fi
 
-# remove htdocs directory, left from a previous script execution
-rm -rf htdocs
-
-# for all C demos executables:
-(cd examples/c; make check)
+EXDIR=htdocs/examples-data
+pushd htdocsgen/install/share/plplot*/examples/c
+export cexamples_dir=`pwd`
+make
+popd
 
 # hack, x20c needs lena in the current directory
 cp examples/c/lena.pgm .
 
-EXDIR=htdocs/examples-data
-TTFDIR=/usr/share/fonts/truetype
+for exe in 01 02 03 04 05 06 07 08 09 10 11 12 13 15 16 18 19 20 21 22 \
+23 24 25 26 27 28; do
 
-for exe in 01 02 03 04 05 06 07 08 09 10 11 12 13 15 16 18 19 20 21 22 23 24 25; do
-
-    echo Working on example ${exe}
+    if test $exe = 08 -o $exe = 16 -o $exe = 20; then
+      # No cairo graphics AA (Yep, 1 turns it off).
+      DRIVEROPT='-drvopt graphics_anti_aliasing=1'
+    else
+      if test $exe = 09 -o $exe = 21; then
+        # Text clipping.
+        DRIVEROPT='-drvopt text_clipping=1'
+      else
+        # Otherwise use default graphics AA which is full AA 
+        DRIVEROPT=
+      fi
+    fi
+    echo Working on example ${exe} using DRIVEROPT of $DRIVEROPT
 
     # generate standard and preview size images
-    if test $exe != 24 ; then
-        if test $exe = 02 ; then
-            SMOOTH=0
-	else
-	    SMOOTH=1
-	fi
-        examples/c/x${exe}c -dev png -drvopt text,smooth=$SMOOTH \
-	    -o x${exe} -fam -fflen 2;
-        examples/c/x${exe}c -dev png -drvopt text,smooth=$SMOOTH \
-	    -o prev-x${exe} -fam -fflen 2 -geometry 200x150;
-    else
-      # Temporary workaround for CTL problem with -dev png
-      PLPLOT_FREETYPE_SANS_FONT=$TTFDIR/arphic/bkai00mp.ttf		    \
-      PLPLOT_FREETYPE_SERIF_FONT=$TTFDIR/freefont/FreeSerif.ttf 	    \
-      PLPLOT_FREETYPE_MONO_FONT=$TTFDIR/ttf-indic-fonts/lohit_hi.ttf	    \
-      PLPLOT_FREETYPE_SCRIPT_FONT=$TTFDIR/unfonts/UnBatang.ttf		    \
-      PLPLOT_FREETYPE_SYMBOL_FONT=$TTFDIR/ttf-bangla-fonts/JamrulNormal.ttf \
-      examples/c/x24c -dev psttfc -o test.ps
-      convert -rotate 90 test.ps -geometry 800x600 x24.01.png
-      convert -rotate 90 test.ps -geometry 200x150 prev-x24.01.png
-      rm test.ps
-      # convert command needs the png suffix, but now take it off to
-      # be consistent with script below which puts it back on again.
-      mv x24.01.png x24.01
-      mv prev-x24.01.png prev-x24.01
-    fi
+    $cexamples_dir/x${exe}c -dev pngcairo -o x${exe} -fam -fflen 2 \
+    $DRIVEROPT
+    $cexamples_dir/x${exe}c -dev pngcairo -o prev-x${exe} -fam -fflen 2 \
+    $DRIVEROPT -geometry 200x150
 
     # give png extension
     for i in `ls x${exe}.?? prev-x${exe}.??`; do
@@ -87,12 +84,19 @@ for exe in 01 02 03 04 05 06 07 08 09 10 11 12 13 15 16 18 19 20 21 22 23 24 25;
 
     # move to www directory.
     echo populating www directory demo${exe}
+    rm -rf $EXDIR/demo${exe}
     mkdir -p $EXDIR/demo${exe}
-    mv *${exe}.??.png $EXDIR/demo${exe}/
-    for f in examples/c/x${exe}c.c examples/tcl/x${exe}.tcl		\
-             examples/java/x${exe}.java examples/f77/x${exe}f.f		\
-             bindings/octave/demos/x${exe}c.m				\
-             examples/python/xw${exe}.py examples/c++/x${exe}.cc	\
+    mv *${exe}.??.png $EXDIR/demo${exe}
+    # Note fortran 77 examples grabbed from installed examples.
+    for f in examples/c/x${exe}c.c \
+             examples/tcl/x${exe}.tcl \
+             examples/java/x${exe}.java \
+	     cexamples_dir/../f77/x${exe}f.f \
+	     examples/f95/x${exe}f.f90 \
+	     examples/ada/x${exe}a.adb \
+             bindings/octave/demos/x${exe}c.m \
+             examples/python/xw${exe}.py \
+	     examples/c++/x${exe}.cc \
              examples/perl/x${exe}.pl ; do
         if test -f $f ; then
             cp $f $EXDIR/demo${exe}
@@ -104,8 +108,8 @@ for exe in 01 02 03 04 05 06 07 08 09 10 11 12 13 15 16 18 19 20 21 22 23 24 25;
 
     # rename executables, to avoid browsers trying to execute files
     # instead of showing them.
-    (cd  htdocs/examples-data/demo${exe};
-    for j in *.c *.cc *.f *.m *.tcl *.java *.py *.pl; do
+    (cd  $EXDIR/demo${exe};
+    for j in *.c *.cc *.f *.f90 *.adb *.m *.tcl *.java *.py *.pl; do
 	    mv $j $j-
     done
     )
@@ -142,4 +146,3 @@ echo Changing group permissions of the remote examples directory
 ssh $WWW_HOST chmod -R g=u $WWW_DIR/$EXDIR
 echo Removing the remote tarball
 ssh $WWW_HOST rm -f $WWW_DIR/$TARBALL
- 
