@@ -1,4 +1,4 @@
-/* $Id: plcore.c,v 1.169 2006/05/27 18:16:01 hbabcock Exp $
+/* $Id: plcore.c 7980 2007-10-27 06:29:52Z airwin $
 
 	Central dispatch facility for PLplot.
 	Also contains the PLplot main data structures, external access
@@ -160,8 +160,9 @@ plP_tidy(void)
 
     (*plsc->dispatch_table->pl_tidy) ((struct PLStream_struct *) plsc);
 
-    if (plsc->plbuf_write)
+    if (plsc->plbuf_write) {
 	plbuf_tidy(plsc);
+    }
 
     plsc->OutFile = NULL;
 
@@ -197,7 +198,7 @@ plP_esc(PLINT op, void *ptr)
       difilt(&(args->x),&(args->y),1,&clpxmi,&clpxma,&clpymi,&clpyma);
     }
   }
-  
+
   (*plsc->dispatch_table->pl_esc) ((struct PLStream_struct *) plsc, op, ptr);
 }
 
@@ -496,27 +497,27 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 
 	 /* Now process the text string */
 
-	 if (string!=NULL) {        /* If the string isn't blank, then we will 
-                                     * continue 
+	 if (string!=NULL) {        /* If the string isn't blank, then we will
+                                     * continue
 				     */
 
-	    len=strlen(string);     /* this length is only used in the loop 
-				     * counter, we will work out the length of 
+	    len=strlen(string);     /* this length is only used in the loop
+				     * counter, we will work out the length of
 				     * the unicode string as we go */
 	    plgesc(&esc);
 	
-	    /* At this stage we will do some translations into unicode, like 
-	     * conversion to Greek , and will save other translations such as 
-	     * superscript for the driver to do later on. As we move through 
+	    /* At this stage we will do some translations into unicode, like
+	     * conversion to Greek , and will save other translations such as
+	     * superscript for the driver to do later on. As we move through
 	     * the string and do the translations, we will get
-	     * rid of the esc character sequence, just replacing it with 
+	     * rid of the esc character sequence, just replacing it with
 	     * unicode.
 	     */
 	
-	    /* Obtain FCI (font characterization integer) for start of 
+	    /* Obtain FCI (font characterization integer) for start of
 	     * string. */
 	    plgfci(&fci);
-	    for (j=i=0;i<len;i++) {    /* Walk through the string, and convert 
+	    for (j=i=0;i<len;i++) {    /* Walk through the string, and convert
 					* some stuff to unicode on the fly */
 	       skip=0;
 	
@@ -532,11 +533,11 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 		     unicode_buffer[j++] = \
 		       (PLUNICODE)hershey_to_unicode_lookup_table[idx].Unicode;
 
-		     /* if unicode_buffer[j-1] corresponds to the escape 
-		      * character must unescape it by appending one more.  
-		      * This will probably always be necessary since it is 
-		      * likely unicode_buffer will always have to contain 
-		      * escape characters that are interpreted by the device 
+		     /* if unicode_buffer[j-1] corresponds to the escape
+		      * character must unescape it by appending one more.
+		      * This will probably always be necessary since it is
+		      * likely unicode_buffer will always have to contain
+		      * escape characters that are interpreted by the device
 		      * driver.
 		      */
 		     if (unicode_buffer[j-1]==esc) unicode_buffer[j++]=esc;
@@ -552,11 +553,11 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 		     plP_hex2fci(PL_FCI_SYMBOL, PL_FCI_FAMILY, &fci);
 		     unicode_buffer[j++]= fci;
 		     unicode_buffer[j++]=code;
-		     /* if unicode_buffer[j-1] corresponds to the escape 
-		      * character must unescape it by appending one more.  
-		      * This will probably always be necessary since it is 
-		      * likely unicode_buffer will always have to contain 
-		      * escape characters that are interpreted by the device 
+		     /* if unicode_buffer[j-1] corresponds to the escape
+		      * character must unescape it by appending one more.
+		      * This will probably always be necessary since it is
+		      * likely unicode_buffer will always have to contain
+		      * escape characters that are interpreted by the device
 		      * driver.
 		      */
 		     if (unicode_buffer[j-1]==esc) unicode_buffer[j++]=esc;
@@ -602,8 +603,8 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 		
 		   case 'f':  /* Deprecated Hershey-style font change*/
 		   case 'F':  /* Deprecated Hershey-style font change*/
-		     /* We implement an approximate response here so that 
-		      * reasonable results are obtained for unicode fonts, 
+		     /* We implement an approximate response here so that
+		      * reasonable results are obtained for unicode fonts,
 		      * but this method is deprecated and the #<nnn> or
 		      * #<command string> methods should be used instead
 		      * to change unicode fonts in mid-string.
@@ -650,15 +651,15 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 			unicode_buffer[j++] = \
 			  (PLUNICODE)hershey_to_unicode_lookup_table[idx].Unicode;
 			i+=2;
-			skip=1;  /* skip is set if we have copied something 
+			skip=1;  /* skip is set if we have copied something
 				  * into the unicode table */
 		     }
 		     else {
-			/* Use "unknown" unicode character if string[i+2] 
+			/* Use "unknown" unicode character if string[i+2]
 			 * is not in the Greek array.*/
 			unicode_buffer[j++]=(PLUNICODE)0x00;
 			i+=2;
-			skip=1;  /* skip is set if we have copied something 
+			skip=1;  /* skip is set if we have copied something
 				  * into  the unicode table */
 		     }
 		     fci = fcisave;
@@ -685,7 +686,7 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
                   unicode_buffer [j] = unichar;
                   i += ptr - (string + i) - 1;
 
-		  /* Search for escesc (an unescaped escape) in the input 
+		  /* Search for escesc (an unescaped escape) in the input
 		   * string and adjust unicode_buffer accordingly).
 		   */
 		  if (unicode_buffer[j] == esc && string[i+1] == esc) {
@@ -696,11 +697,11 @@ plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 	       j++;
 	    }
 	    if (j > 0) {
-	       args.unicode_array_len=j; /* Much easier to set the length than 
+	       args.unicode_array_len=j; /* Much easier to set the length than
 					  * work it out later :-) */
-	       args.unicode_array=&unicode_buffer[0]; /* Get address of the 
-						       * unicode buffer (even 
-						       * though it is 
+	       args.unicode_array=&unicode_buffer[0]; /* Get address of the
+						       * unicode buffer (even
+						       * though it is
 						       * currently  static) */
 	    } else
 	      /* Don't print anything, if there is no unicode to print! */
@@ -730,7 +731,7 @@ utf8_to_ucs4(const char *ptr, PLUNICODE *unichar)
    char tmp;
    int isFirst = 1;
    int cnt = 0;
-   
+
    do {
       /* Get next character in string */
       tmp = *ptr++;
@@ -797,7 +798,7 @@ ucs4_to_utf8(PLUNICODE unichar, char *ptr)
     *tmp = (unsigned char) 0xe0 | (unichar >> 12);
     tmp++;
     *tmp = (unsigned char) 0x80 | ((unichar >> 6) & 0x3f);
-    tmp++;      
+    tmp++;
     *tmp = (unsigned char) 0x80 | (unichar & 0x3f);
     tmp++;
     len = 3;
@@ -806,9 +807,9 @@ ucs4_to_utf8(PLUNICODE unichar, char *ptr)
     *tmp = (unsigned char) 0xf0 | (unichar >> 18);
     tmp++;
     *tmp = (unsigned char) 0x80 | ((unichar >> 12) & 0x3f);
-    tmp++;      
+    tmp++;
     *tmp = (unsigned char) 0x80 | ((unichar >> 6) & 0x3f);
-    tmp++;      
+    tmp++;
     *tmp = (unsigned char) 0x80 | (unichar & 0x3f);
     tmp++;
     len = 4;
@@ -1815,6 +1816,9 @@ c_plend1(void)
     free_mem(plsc->geometry);
     free_mem(plsc->dev);
     free_mem(plsc->BaseName);
+#ifndef BUFFERED_FILE
+    free_mem(plsc->plbuf_buffer);
+#endif
     if (plsc->program) free_mem(plsc->program);
     if (plsc->server_name) free_mem(plsc->server_name);
     if (plsc->server_host) free_mem(plsc->server_host);
@@ -1992,7 +1996,15 @@ c_plcpstrm(PLINT iplsr, PLINT flags)
 /* Plot buffer -- need to copy file pointer so that plreplot() works */
 /* This also prevents inadvertent writes into the plot buffer */
 
+#ifdef BUFFERED_FILE
     plsc->plbufFile = plsr->plbufFile;
+#else
+    plsc->plbuf_buffer = plsr->plbuf_buffer;
+    plsc->plbuf_buffer_grow = plsr->plbuf_buffer_grow;
+    plsc->plbuf_buffer_size = plsr->plbuf_buffer_size;
+    plsc->plbuf_top = plsr->plbuf_top;
+    plsc->plbuf_readpos = plsr->plbuf_readpos;
+#endif
 
 /* Driver interface */
 /* Transformation must be recalculated in current driver coordinates */
@@ -2027,7 +2039,11 @@ c_plcpstrm(PLINT iplsr, PLINT flags)
     if (plsc->cmap0 != NULL)
 	free((void *) plsc->cmap0);
 
-    plsc->cmap0 = (PLColor *) calloc(1, plsc->ncol0 * sizeof(PLColor));
+    if ((plsc->cmap0 = (PLColor *) calloc(1, plsc->ncol0 * sizeof(PLColor)))==NULL)
+      {
+        plexit("c_plcpstrm: Insufficient memory");
+      }
+
     for (i = 0; i < plsc->ncol0; i++)
 	pl_cpcolor(&plsc->cmap0[i], &plsr->cmap0[i]);
 
@@ -2038,7 +2054,11 @@ c_plcpstrm(PLINT iplsr, PLINT flags)
     if (plsc->cmap1 != NULL)
 	free((void *) plsc->cmap1);
 
-    plsc->cmap1 = (PLColor *) calloc(1, plsc->ncol1 * sizeof(PLColor));
+    if ((plsc->cmap1 = (PLColor *) calloc(1, plsc->ncol1 * sizeof(PLColor)))==NULL)
+      {
+        plexit("c_plcpstrm: Insufficient memory");
+      }
+
     for (i = 0; i < plsc->ncol1; i++)
 	pl_cpcolor(&plsc->cmap1[i], &plsr->cmap1[i]);
 
@@ -2083,7 +2103,7 @@ pllib_devinit()
     plsc->dispatch_table = dispatch_table[plsc->device - 1];
 }
 
-int plInBuildTree()
+int PLDLLIMPEXP plInBuildTree()
 {
   static int inited = 0;
   static int inBuildTree = 0;
@@ -2095,7 +2115,7 @@ int plInBuildTree()
    name is _getcwd, this is a problem in the case of DLLs, like with
    the Java bindings.
 */
-#ifdef WIN32
+#if defined(WIN32) && !defined(__BORLANDC__)
 #define getcwd _getcwd
 #endif
 
@@ -2110,7 +2130,7 @@ int plInBuildTree()
 
 #ifdef ENABLE_DYNDRIVERS
 
-static char*
+char*
 plGetDrvDir ()
 {
     char* drvdir;
@@ -2241,8 +2261,11 @@ plInitDispatchTable()
 #endif
 
 /* Allocate space for the dispatch table. */
-    dispatch_table = (PLDispatchTable **)
-	malloc( (nplstaticdevices + npldynamicdevices) * sizeof(PLDispatchTable *) );
+    if ((dispatch_table = (PLDispatchTable **)
+	malloc( (nplstaticdevices + npldynamicdevices) * sizeof(PLDispatchTable *) ))==NULL)
+     {
+       plexit("plInitDispatchTable: Insufficient memory");
+     }
 
 /* Initialize the dispatch table entries for the static devices by calling
    the dispatch table initialization function for each static device.  This
@@ -2251,7 +2274,10 @@ plInitDispatchTable()
 
     for( n=0; n < nplstaticdevices; n++ )
     {
-        dispatch_table[n] = (PLDispatchTable *)malloc( sizeof(PLDispatchTable) );
+        if ((dispatch_table[n] = (PLDispatchTable *)malloc( sizeof(PLDispatchTable) ))==NULL)
+           {
+             plexit("plInitDispatchTable: Insufficient memory");
+           }
 
         (*static_device_initializers[n])( dispatch_table[n] );
     }
@@ -2262,8 +2288,11 @@ plInitDispatchTable()
 /* Allocate space for the device and driver specs.  We may not use all of
  * these driver descriptors, but we obviously won't need more drivers than
  * devices... */
-    loadable_device_list = malloc( npldynamicdevices * sizeof(PLLoadableDevice) );
-    loadable_driver_list = malloc( npldynamicdevices * sizeof(PLLoadableDriver) );
+    if (((loadable_device_list = malloc( npldynamicdevices * sizeof(PLLoadableDevice) ))==NULL)||
+        ((loadable_driver_list = malloc( npldynamicdevices * sizeof(PLLoadableDriver) ))==NULL))
+       {
+         plexit("plInitDispatchTable: Insufficient memory");
+       }
 
     rewind( fp_drvdb );
 
@@ -2288,7 +2317,10 @@ plInitDispatchTable()
 
         n = npldrivers++;
 
-        dispatch_table[n] = malloc( sizeof(PLDispatchTable) );
+        if ((dispatch_table[n] = malloc( sizeof(PLDispatchTable) ))==NULL)
+          {
+            plexit("plInitDispatchTable: Insufficient memory");
+          }
 
     /* Fill in the dispatch table entries. */
         dispatch_table[n]->pl_MenuStr = plstrdup(devdesc);
@@ -2403,9 +2435,7 @@ plSelectDev()
 	    fprintf(stdout, "\nEnter device number or keyword (stream %d): ",
 		   (int) ipls);
 
-	if(!(fgets(response, sizeof(response), stdin))){
-		plexit("plSelectDev: Failed to process response.");
-	}
+	plio_fgets(response, sizeof(response), stdin);
 
     /* First check to see if device keyword was entered. */
     /* Final "\n" in response messes things up, so ignore it.  */
@@ -2554,7 +2584,11 @@ c_plfontld(PLINT ifont)
 void
 c_plreplot(void)
 {
+#ifdef BUFFERED_FILE
     if (plsc->plbufFile != NULL) {
+#else
+    if (plsc->plbuf_buffer != NULL) {
+#endif
 	plRemakePlot(plsc);
     }
     else {
@@ -3395,8 +3429,12 @@ plP_image(short *x, short *y, unsigned short *z , PLINT nx, PLINT ny, PLFLT xmin
   if (plsc->difilt) { /* isn't this odd? when replaying the plot buffer, e.g., when resizing the window, difilt() is caled again! the plot buffer should already contain the transformed data--it would save a lot of time! (and allow for differently oriented plots when in multiplot mode) */
     PLINT clpxmi, clpxma, clpymi, clpyma;
 
-    xscl = (short *) malloc(nx*ny*sizeof(short));
-    yscl = (short *) malloc(nx*ny*sizeof(short));
+    if (((xscl = (short *) malloc(nx*ny*sizeof(short)))==NULL)||
+        ((yscl = (short *) malloc(nx*ny*sizeof(short)))==NULL))
+       {
+         plexit("plP_image: Insufficient memory");
+       }
+
     for (i = 0; i < npts; i++) {
       xscl[i] = x[i];
       yscl[i] = y[i];
