@@ -1,4 +1,4 @@
-/* $Id: plctrl.c,v 1.85 2006/05/27 18:16:56 hbabcock Exp $
+/* $Id: plctrl.c 7815 2007-08-20 23:15:48Z airwin $
 
 	Misc. control routines, like begin, end, exit, change graphics/text
 	mode, change color.  Includes some spillage from plcore.c.  If you
@@ -52,7 +52,7 @@
 /* Static functions */
 
 /* Used by any external init code to suggest a path */
-char* plplotLibDir = 0;
+char PLDLLIMPEXP * plplotLibDir = 0;
 
 static void
 color_set(PLINT i, U_CHAR r, U_CHAR g, U_CHAR b, char *name );
@@ -541,11 +541,17 @@ c_plscmap0n(PLINT ncol0)
 /* Allocate the space */
 
     if (plsc->cmap0 == NULL) {
-	plsc->cmap0 = (PLColor *) calloc(1, size);
+	if ((plsc->cmap0 = (PLColor *) calloc(1, size))==NULL)
+     {
+       plexit("c_plscmap0n: Insufficient memory");
+     }
 	imin = 0;
     }
     else {
-	plsc->cmap0 = (PLColor *) realloc(plsc->cmap0, size);
+	if ((plsc->cmap0 = (PLColor *) realloc(plsc->cmap0, size))==NULL)
+     {
+       plexit("c_plscmap0n: Insufficient memory");
+     }
 	imin = plsc->ncol0;
     }
 
@@ -592,9 +598,19 @@ c_plscmap1n(PLINT ncol1)
 /* Allocate the space */
 
     if (plsc->ncol1 > 0)
-	plsc->cmap1 = (PLColor *) realloc(plsc->cmap1, size);
+      {
+        if ((plsc->cmap1 = (PLColor *) realloc(plsc->cmap1, size))==NULL)
+          {
+            plexit("c_plscmap1n: Insufficient memory");
+          }
+      }
     else
-	plsc->cmap1 = (PLColor *) calloc(ncol, sizeof(PLColor));
+      {
+        if ((plsc->cmap1 = (PLColor *) calloc(ncol, sizeof(PLColor)))==NULL)
+          {
+            plexit("c_plscmap1n: Insufficient memory");
+          }
+      }
 
 /* Fill in default entries */
 
@@ -1000,7 +1016,7 @@ plabort(char *errormsg)
 	*(plsc->errcode) = 1;
 
     if (plsc->errmsg != NULL) {
-	sprintf(plsc->errmsg, "\n*** PLPLOT ERROR ***\n");
+	sprintf(plsc->errmsg, "\n*** PLPLOT ERROR, ABORTING OPERATION ***\n");
 	if (*errormsg != '\0')
 	    sprintf(plsc->errmsg, "%s, aborting operation\n", errormsg);
 
@@ -1012,7 +1028,7 @@ plabort(char *errormsg)
 	    pltext();
 	}
 
-	fprintf(stderr, "\n*** PLPLOT ERROR ***\n");
+	fprintf(stderr, "\n*** PLPLOT ERROR, ABORTING OPERATION ***\n");
 	if (*errormsg != '\0')
 	    fprintf(stderr, "%s, aborting operation\n", errormsg);
 
@@ -1056,7 +1072,7 @@ plexit(char *errormsg)
 
     plsc->nopause = 1;
     if (*errormsg != '\0') {
-	fprintf(stderr, "\n*** PLPLOT ERROR ***\n");
+	fprintf(stderr, "\n*** PLPLOT ERROR, IMMEDIATE EXIT ***\n");
 	fprintf(stderr, "%s\n", errormsg);
     }
     plend();
@@ -1431,7 +1447,10 @@ plGetName(char *dir, char *subdir, char *filename, char **filespec)
     free_mem(*filespec);
     lfilespec = 10;
     lfilespec = strlen(dir) + strlen(subdir) + strlen(filename) + 10;
-    *filespec = (char *) malloc(lfilespec);
+    if ((*filespec = (char *) malloc(lfilespec))==NULL)
+      {
+        plexit("plGetName: Insufficient memory");
+      }
 
     strcpy(*filespec, dir);
 
@@ -1531,9 +1550,7 @@ plOpenFile(PLStream *pls)
 	if (pls->FileName == NULL) {
 	    do {
 		fprintf(stdout, "Enter graphics output file name: ");
-		if(!(fgets(line, sizeof(line), stdin))){
-			plexit("plOpenFile: Failed to process response.");
-		}
+		plio_fgets(line, sizeof(line), stdin);
 		len = strlen(line);
 		if (len)
 		    len--;
@@ -1577,21 +1594,26 @@ plP_getmember(PLStream *pls)
 {
     char tmp[256];
     char prefix[256];
-    char* suffix; 
+    char* suffix;
 
     if (pls->FileName == NULL)
-	pls->FileName = (char *) malloc(10 + strlen(pls->BaseName));
+      {
+        if ((pls->FileName = (char *) malloc(10 + strlen(pls->BaseName)))==NULL)
+          {
+            plexit("plP_getmember: Insufficient memory");
+          }
+      }
 
     suffix = strstr (pls->BaseName, "%n");
 
-    if (suffix == NULL) 
+    if (suffix == NULL)
       sprintf (tmp, "%s.%%0%1ii", pls->BaseName, (int) pls->fflen);
     else {
       strncpy (prefix, pls->BaseName, 256);
       prefix [suffix - pls->BaseName] = 0;
       sprintf (tmp, "%s%%0%1ii%s", prefix, (int) pls->fflen, suffix + 2);
     }
-  
+
     sprintf(pls->FileName, tmp, pls->member);
 }
 
@@ -1610,14 +1632,20 @@ plP_sfnam(PLStream *pls, const char *fnam)
     if (pls->FileName != NULL)
 	free((void *) pls->FileName);
 
-    pls->FileName = (char *) malloc(10 + strlen(fnam));
+    if ((pls->FileName = (char *) malloc(10 + strlen(fnam)))==NULL)
+      {
+        plexit("plP_sfnam: Insufficient memory");
+      }
 
     strcpy(pls->FileName, fnam);
 
     if (pls->BaseName != NULL)
 	free((void *) pls->BaseName);
 
-    pls->BaseName = (char *) malloc(10 + strlen(fnam));
+    if ((pls->BaseName = (char *) malloc(10 + strlen(fnam)))==NULL)
+      {
+        plexit("plP_sfnam: Insufficient memory");
+      }
 
     strcpy(pls->BaseName, fnam);
 }
@@ -1768,9 +1796,7 @@ plGetInt(char *s)
 
     while (i++ < 10) {
 	fprintf(stdout, s);
-	if(!(fgets(line, sizeof(line), stdin))){
-		plexit("plGetInt: Failed to process response.");
-	}
+	plio_fgets(line, sizeof(line), stdin);
 
 #ifdef MSDOS
 	m = atoi(line);
@@ -1801,9 +1827,7 @@ plGetFlt(char *s)
 
     while (i++ < 10) {
 	fprintf(stdout, s);
-	if(!(fgets(line, sizeof(line), stdin))){
-		plexit("plOpenFile: Failed to process response.");
-	}
+	plio_fgets(line, sizeof(line), stdin);
 
 #ifdef MSDOS
 	m = atof(line);
@@ -1827,7 +1851,7 @@ plGetFlt(char *s)
  * Caller responsible for freeing the allocated memory.
 \*--------------------------------------------------------------------------*/
 
-char *
+char PLDLLIMPEXP *
 plstrdup(const char *src)
 {
     char *dest = (char *) malloc( (strlen(src) + 1) * sizeof(char) );
