@@ -1,4 +1,4 @@
-/* $Id: plfreetype.c,v 1.51 2006/05/27 19:04:28 hbabcock Exp $
+/* $Id: plfreetype.c 7801 2007-08-15 18:50:21Z andrewross $
  *
  * Copyright (C) 2002, 2004, 2005  Andrew Roach
  * Copyright (C) 2002  Maurice LeBrun
@@ -65,25 +65,25 @@
  *
  */
 
-#ifndef WIN32
-#ifndef MINGW
-#include <unistd.h>
+#if !defined(WIN32) || defined(__GNUC__)
+  #include <unistd.h>
 #else
-#define F_OK 1
-#include <stdio.h>
-int access( char *filename, int flag ) {
-    FILE *infile ;
-    infile = fopen( filename, "r" ) ;
-    if ( infile != NULL ) {
-        fclose(infile) ;
-        return 0 ;
-    } else {
-        return 1 ;
-    }
-}
-#endif
+  #define F_OK 1
+  #include <stdio.h>
+  int access( char *filename, int flag ) {
+      FILE *infile ;
+      infile = fopen( filename, "r" ) ;
+      if ( infile != NULL ) {
+          fclose(infile) ;
+          return 0 ;
+      } else {
+          return 1 ;
+      }
+  }
 #endif
 
+#define makeunixslash( b ) do { char *I; for (I=b;*I!=0;*I++) if (*I=='\\') *I='/';}while(0)
+
 #include "plDevs.h"
 #include "plplotP.h"
 #include "drivers.h"
@@ -393,17 +393,78 @@ FT_PlotChar(PLStream *pls, FT_Data *FT, FT_GlyphSlot slot,
     unsigned char bittest;
     short i,k,j;
     int n=slot->bitmap.pitch;
+    int current_pixel_colour;
+    int R,G,B;
+    PLFLT alpha_a,alpha_b;
+    int xx;
+    short imin, imax, kmin, kmax;
+
+    /* Corners of the clipping rectangle */
+    PLINT clipxmin, clipymin, clipxmax,clipymax, tmp;
+    PLINT clpxmi, clpxma, clpymi, clpyma;
+
+    /* Convert clipping box into normal coordinates */
+    clipxmin=pls->clpxmi;
+    clipxmax=pls->clpxma;
+    clipymin=pls->clpymi;
+    clipymax=pls->clpyma;
+
+    if (plsc->difilt) {
+      difilt(&clipxmin,&clipymin,1,&clpxmi,&clpxma,&clpymi,&clpyma);
+      difilt(&clipxmax,&clipymax,1,&clpxmi,&clpxma,&clpymi,&clpyma);
+    }
+
+
+    if( FT->scale!=0.0 ) {  /* scale was set */
+      clipxmin = clipxmin/FT->scale;
+      clipxmax = clipxmax/FT->scale;
+      if (FT->invert_y==1) {
+	clipymin=FT->ymax-(clipymin/FT->scale);
+	clipymax=FT->ymax-(clipymax/FT->scale);
+      }
+      else {
+	clipymin=clipymin/FT->scale;
+	clipymax=clipymax/FT->scale;
+      }
+    } 
+    else {
+      clipxmin=clipxmin/FT->scalex;
+      clipxmax=clipxmax/FT->scalex;
+      
+      if (FT->invert_y==1) {
+	clipymin=FT->ymax-(clipymin/FT->scaley);
+	clipymax=FT->ymax-(clipymax/FT->scaley);
+      } 
+      else {
+	clipymin=clipymin/FT->scaley;
+	clipymax=clipymax/FT->scaley;
+      }
+    }
+    if (clipxmin > clipxmax) {
+      tmp = clipxmax;
+      clipxmax = clipxmin;
+      clipxmin = tmp;
+    }
+    if (clipymin > clipymax) {
+      tmp = clipymax;
+      clipymax = clipymin;
+      clipymin = tmp;
+    }
 
     if ((slot->bitmap.pixel_mode==ft_pixel_mode_mono)||(pls->icol0==0)) {
 	x+=slot->bitmap_left;
 	y-=slot->bitmap_top;
 
-	for(i=0;i<slot->bitmap.rows;i++) {
+        imin = MAX(0,clipymin-y);
+        imax = MIN(slot->bitmap.rows,clipymax-y);
+	for(i=imin;i<imax;i++) {
 	    for (k=0;k<n;k++) {
 		bittest=128;
 		for (j=0;j<8;j++) {
 		    if ((bittest&(unsigned char)slot->bitmap.buffer[(i*n)+k])==bittest)
-			FT->pixel(pls, x+(k*8)+j, y+i);
+			xx = x+(k*8)+j;
+			if ( (xx >= clipxmin) && (xx <= clipxmax) )
+			  FT->pixel(pls, xx, y+i);
 		    bittest>>=1;
 		}
             }
@@ -411,26 +472,72 @@ FT_PlotChar(PLStream *pls, FT_Data *FT, FT_GlyphSlot slot,
     }
 
 /* this is the anti-aliased stuff */
-/* At present only one anti-aliased mode is supported, using a
-   fixed background and palette. That might change eventually */
 
     else {
-	x+=slot->bitmap_left;
-	y-=slot->bitmap_top;
+           x+=slot->bitmap_left;
+           y-=slot->bitmap_top;
 
-	for(i=0;i<slot->bitmap.rows;i++) {
-	    for (k=0;k<slot->bitmap.width;k++) {
-		FT->shade=(slot->bitmap.buffer[(i*slot->bitmap.width)+k]);
-		if (FT->shade>0) {
-		    FT->col_idx=FT->ncol0_width-((FT->ncol0_width*FT->shade)/255);
-		    FT->last_icol0=pls->icol0;
-		    plcol0(pls->icol0+(FT->col_idx*(FT->ncol0_org-1)));
-		    FT->pixel(pls,x+k,y+i);
-		    plcol0(FT->last_icol0);
-                }
-	    }
+          imin = MAX(0,clipymin-y);
+          imax = MIN(slot->bitmap.rows,clipymax-y);
+          kmin = MAX(0,clipxmin-x);
+          kmax = MIN(slot->bitmap.width,clipxmax-x);
+           for(i=imin;i<imax;i++)
+              {
+                for (k=kmin;k<kmax;k++)
+                  {
+                    FT->shade=(slot->bitmap.buffer[(i*slot->bitmap.width)+k]);
+                    if (FT->shade>0)
+                      {
+                        if ((FT->BLENDED_ANTIALIASING==1)&&(FT->read_pixel!=NULL))
+                          /* The New anti-aliasing technique */
+                          {
+                            if (FT->shade==255)
+                              {
+                                FT->pixel(pls,x+k,y+i);
+                              }
+                            else
+                              {
+                                current_pixel_colour=FT->read_pixel(pls,x+k,y+i);
+
+                                G=GetGValue(current_pixel_colour);
+                                R=GetRValue(current_pixel_colour);
+                                B=GetBValue(current_pixel_colour);
+                                alpha_a=(float)FT->shade/255.0;
+
+                                /* alpha_b=1.0-alpha_a;
+                                R=(plsc->cmap0[pls->icol0].r*alpha_a)+(R*alpha_b);
+                                G=(plsc->cmap0[pls->icol0].g*alpha_a)+(G*alpha_b);
+                                B=(plsc->cmap0[pls->icol0].b*alpha_a)+(B*alpha_b);
+                                 */
+
+                                /*  This next bit of code is, I *think*, computationally
+                                 *  more efficient than the bit above. It results in
+                                 *  an indistinguishable plot, but file sizes are different
+                                 *  suggesting subtle variations doubtless caused by rounding
+                                 *  and/or floating point conversions. Questions are - which is
+                                 *  better ? Which is more "correct" ? Does it make a difference ?
+                                 *  Is one faster than the other so that you'd ever notice ?
+                                 */
+
+                                R=(((plsc->cmap0[pls->icol0].r-R)*alpha_a)+R);
+                                G=(((plsc->cmap0[pls->icol0].g-G)*alpha_a)+G);
+                                B=(((plsc->cmap0[pls->icol0].b-B)*alpha_a)+B);
+
+                                FT->set_pixel(pls,x+k,y+i,RGB(R>255 ? 255 : R,G>255 ? 255 : G,B>255 ? 255 : B));
+                              }
+                          }
+                        else /* The old anti-aliasing technique */
+                          {
+                            FT->col_idx=FT->ncol0_width-((FT->ncol0_width*FT->shade)/255);
+                            FT->last_icol0=pls->icol0;
+                            plcol0(pls->icol0+(FT->col_idx*(FT->ncol0_org-1)));
+                            FT->pixel(pls,x+k,y+i);
+                            plcol0(FT->last_icol0);
+                          }
+                      }
+                  }
+              }
         }
-    }
 }
 
 /*----------------------------------------------------------------------*\
@@ -485,10 +592,15 @@ void plD_FreeType_init(PLStream *pls)
 #if defined(MSDOS) || defined(WIN32)
     static char *default_font_names[]={"arial.ttf","times.ttf","timesi.ttf","arial.ttf",
 				       "symbol.ttf"};
+    char WINDIR_PATH[255];
+    char *b;
+    b=getenv("WINDIR");
+    strcpy(WINDIR_PATH,b);
 #else
     const char *default_unix_font_dir=PL_FREETYPE_FONT_DIR;
 #endif
 
+
     if (pls->FT) {
 	plwarn("Freetype seems already to have been initialised!");
 	return;
@@ -517,16 +629,33 @@ void plD_FreeType_init(PLStream *pls)
  * At present, it only looks in two places, on one drive. I might change this
  * soon.
  */
-    if (access("c:\\windows\\fonts\\arial.ttf", F_OK)==0) {
-        strcpy(font_dir,"c:/windows/fonts/");
-    }
-    else if ( access("c:\\windows\\system\\arial.ttf", F_OK)==0) {
-        strcpy(font_dir,"c:/windows/system/");
+    if (WINDIR_PATH==NULL)
+    {
+        if (access("c:\\windows\\fonts\\arial.ttf", F_OK)==0) {
+            strcpy(font_dir,"c:/windows/fonts/");
+        }
+        else if ( access("c:\\windows\\system\\arial.ttf", F_OK)==0) {
+            strcpy(font_dir,"c:/windows/system/");
+        }
+        else
+        plwarn("Could not find font path; I sure hope you have defined fonts manually !");
     }
     else
-	plwarn("Could not find font path; I sure hope you have defined fonts manually !");
+    {
+      strcat(WINDIR_PATH,"\\fonts\\arial.ttf");
+      if (access(WINDIR_PATH, F_OK)==0)
+        {
+          b=strrchr(WINDIR_PATH,'\\');
+          b++;
+          *b=0;
+          makeunixslash(WINDIR_PATH);
+          strcpy(font_dir,WINDIR_PATH);
+        }
+      else
+        plwarn("Could not find font path; I sure hope you have defined fonts manually !");
+    }
 
-    fprintf( stderr, "%s\n", font_dir ) ;
+    if (pls->debug) fprintf( stderr, "%s\n", font_dir ) ;
 #else
 
 /*
@@ -813,13 +942,21 @@ if ((args->string!=NULL)||(args->unicode_array_len>0))
  *   Convert into normal coordinates from virtual coordinates
  */
 
-  x=args->x/FT->scale;
+  if( FT->scale!=0.0 ) {  /* scale was set */
+      x=args->x/FT->scale;
 
- if (FT->invert_y==1)
-    y=FT->ymax-(args->y/FT->scale);
- else
-    y=args->y/FT->scale;
+     if (FT->invert_y==1)
+        y=FT->ymax-(args->y/FT->scale);
+     else
+        y=args->y/FT->scale;
+  } else {
+      x=args->x/FT->scalex;
 
+     if (FT->invert_y==1)
+        y=FT->ymax-(args->y/FT->scaley);
+     else
+        y=args->y/FT->scaley;
+ }
 
  /*          Adjust for the justification and character height
   *
@@ -919,7 +1056,7 @@ void plD_FreeType_Destroy(PLStream *pls)
     extern int FT_Done_Library( FT_Library  library );
 
     if (FT) {
-	if (FT->smooth_text==1) plscmap0n(FT->ncol0_org);
+	if ((FT->smooth_text==1)&&(FT->BLENDED_ANTIALIASING==0)) plscmap0n(FT->ncol0_org);
 
 	FT_Done_Library(FT->library);
 	free(pls->FT);
@@ -1012,13 +1149,21 @@ void plD_render_freetype_sym (PLStream *pls, EscText *args)
     FT_Vector  adjust;
     PLUNICODE fci;
 
-    x=args->x/FT->scale;
+    if( FT->scale!=0.0 ) {  /* scale was set */
+      x=args->x/FT->scale;
 
- if (FT->invert_y==1)
-    y=FT->ymax-(args->y/FT->scale);
- else
-    y=args->y/FT->scale;
+      if (FT->invert_y==1)
+        y=FT->ymax-(args->y/FT->scale);
+      else
+        y=args->y/FT->scale;
+    } else {
+      x=args->x/FT->scalex;
 
+      if (FT->invert_y==1)
+        y=FT->ymax-(args->y/FT->scaley);
+      else
+        y=args->y/FT->scaley;
+    }
 
 
 /*
