@@ -1,4 +1,4 @@
-/* $Id: plot3d.c,v 1.66 2006/05/12 23:53:41 airwin Exp $
+/* $Id: plot3d.c 7876 2007-09-20 01:51:19Z hbabcock $
 
 	3d plot routines.
 
@@ -483,13 +483,14 @@ c_plsurf3dl(PLFLT *x, PLFLT *y, PLFLT **z, PLINT nx, PLINT ny,
   if (clevel != NULL && opt & BASE_CONT) {
 #define NPTS 100
     int np = NPTS;
+    PLFLT **zstore;
+    PLcGrid2 cgrid2;
     PLFLT *zz = (PLFLT *) malloc(NPTS*sizeof(PLFLT));
+    if (zz==NULL) plexit("plsurf3dl: Insufficient memory");
 
     /* get the contour lines */
 
     /* prepare cont_store input */
-    PLFLT **zstore;
-    PLcGrid2 cgrid2;
     cgrid2.nx = nx;
     cgrid2.ny = ny;
     plAlloc2dGrid(&cgrid2.xg, nx, ny);
@@ -529,7 +530,10 @@ c_plsurf3dl(PLFLT *x, PLFLT *y, PLFLT **z, PLINT nx, PLINT ny,
       do { /* there are several lines that make up the contour */
 	if (cline->npts > np) {
 	  np = cline->npts;
-	  zz = (PLFLT *) realloc(zz, np*sizeof(PLFLT));
+	  if ((zz = (PLFLT *) realloc(zz, np*sizeof(PLFLT)))==NULL)
+  	     {
+          plexit("c_plsurf3dl: Insufficient memory");
+        }
 	}
 	for (j=0; j<cline->npts; j++)
 	  zz[j] = plsc->ranmi;
@@ -787,7 +791,7 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
 	return;
     }
 
-    if (opt < 1) {
+    if ((opt & 3) == 0) {
 	myabort("plot3dcl: Bad option");
 	return;
     }
@@ -855,9 +859,12 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
       }
 
       /* allocate storage for new versions of the input vectors */
-      _x = (PLFLT*)malloc(_nx * sizeof(PLFLT));
-      _y = (PLFLT*)malloc(_ny * sizeof(PLFLT));
-      _z = (PLFLT**)malloc(_nx * sizeof(PLFLT*));
+      if (((_x = (PLFLT*)malloc(_nx * sizeof(PLFLT)))==NULL)||
+          ((_y = (PLFLT*)malloc(_ny * sizeof(PLFLT)))==NULL)||
+          ((_z = (PLFLT**)malloc(_nx * sizeof(PLFLT*)))==NULL))
+          {
+            plexit("c_plot3dcl: Insufficient memory");
+          }
 
       clipped = 1;
 
@@ -873,7 +880,12 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
 
       /* copy the data array so we can interpolate around the edges */
       for(i=0; i<_nx; i++)
-	_z[i] = (PLFLT*)malloc(_ny * sizeof(PLFLT));
+      {
+       if ((_z[i] = (PLFLT*)malloc(_ny * sizeof(PLFLT)))==NULL)
+         {
+           plexit("c_plot3dcl: Insufficient memory");
+         }
+      }
 
       /* interpolation factors for the 4 edges */
       ty0 = (_y[0] - y[iymin]) / (y[iymin+1] - y[iymin]);
@@ -943,7 +955,10 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
    }
 
    if (opt & MAG_COLOR) {     /* If enabled, use magnitude colored wireframe  */
-     ctmp = (PLFLT *) malloc((size_t) (2 * MAX(nx, ny) * sizeof(PLFLT)));
+     if ((ctmp = (PLFLT *) malloc((size_t) (2 * MAX(nx, ny) * sizeof(PLFLT))))==NULL)
+     {
+       plexit("c_plot3dcl: Insufficient memory");
+     }
    } else
      ctmp = NULL;
 
@@ -1032,10 +1047,15 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
 
       PLINT *uu = (PLINT *) malloc(NPTS*sizeof(PLINT));
       PLINT *vv = (PLINT *) malloc(NPTS*sizeof(PLINT));
-
       /* prepare cont_store input */
       PLFLT **zstore;
       PLcGrid2 cgrid2;
+
+      if ((uu==NULL)||(vv==NULL))
+        {
+          plexit("c_plot3dcl: Insufficient memory");
+        }
+
       cgrid2.nx = nx;
       cgrid2.ny = ny;
       plAlloc2dGrid(&cgrid2.xg, nx, ny);
@@ -1070,8 +1090,11 @@ c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
 	  PLFLT tx, ty;
 	  if (cline->npts > np) {
 	    np = cline->npts;
-	    uu = (PLINT *) realloc(uu, np*sizeof(PLINT));
-	    vv = (PLINT *) realloc(vv, np*sizeof(PLINT));
+	    if (((uu = (PLINT *) realloc(uu, np*sizeof(PLINT)))==NULL)||
+	        ((vv = (PLINT *) realloc(vv, np*sizeof(PLINT)))==NULL))
+	        {
+             plexit("c_plot3dcl: Insufficient memory");
+           }
 	  }
 
 	  /* the hidden line plotter plnxtv() only works OK if the x points are in increasing order.
@@ -2341,3 +2364,38 @@ pl3cut(PLINT sx1, PLINT sy1, PLINT sx2, PLINT sy2,
 	*cy = (PLINT) (sy1 + (fb * y21) / fa + .5);
     }
 }
+
+/*--------------------------------------------------------------------------*\
+ * void plRotationShear
+ *
+ * Calculates the rotation and shear angles from a plplot transformation matrix
+ *
+ * N.B. The plot transformation matrix is the product of the following
+ * rotation and shear matrices:
+ *
+ * [c(t) -s(t)] [1 s(p)]
+ * [s(t)  c(t)] [0 c(p)]
+ * 
+ * Where t is the rotation angle and phi is the shear angle.
+\*--------------------------------------------------------------------------*/
+
+void
+plRotationShear(PLFLT *xFormMatrix, PLFLT *rotation, PLFLT *shear)
+{
+  *rotation = acos(xFormMatrix[0]);
+  if(xFormMatrix[2] < 0.0){
+    *rotation = -*rotation;
+  }
+
+  *shear = -asin(xFormMatrix[0]*xFormMatrix[1] +
+		 xFormMatrix[2]*xFormMatrix[3]);
+
+  // Compute the cross product of the vectors [1,0] and [0,1] to 
+  // determine if we need to make a "quadrant 3,4" adjustment
+  // to the shear angle.
+
+  if(xFormMatrix[0]*xFormMatrix[3] - xFormMatrix[1]*xFormMatrix[2] < 0.0){
+    *shear = -(M_PI + *shear);
+  }
+}
+
