@@ -1,4 +1,4 @@
-/* $Id: gnusvga.c,v 1.17 2004/01/27 04:54:07 aroach Exp $
+/* $Id: gnusvga.c 6952 2006-08-08 05:22:52Z aroach $
 */
 
 #include "plDevs.h"
@@ -67,7 +67,7 @@
 #include <keys.h>
 #endif
 
-#include <strings.h>
+#include <string.h>
 
 
 /* Prototypes:  Since GNU CC, we can rest in the safety of ANSI prototyping. */
