@@ -1,4 +1,4 @@
-/* $Id: pltcl.c,v 1.21 2005/04/27 06:44:27 rlaboiss Exp $
+/* $Id: pltcl.c 7114 2006-10-24 04:45:19Z arjenmarkus $
  *
  *	Main program for Tcl-interface to PLplot.  Allows interpretive
  *	execution of plotting primitives without regard to output driver.
@@ -28,6 +28,7 @@
  */
 
 #include "plplotP.h"
+#define USINGPLDLL
 #include "pltcl.h"
 #ifdef HAVE_ITCL
 # ifndef HAVE_ITCLDECLS_H
@@ -139,8 +140,8 @@ AppInit(Tcl_Interp *interp)
  *
  * where "Mod" is the name of the module.
  */
-
     if (Tcl_Init(interp) == TCL_ERROR) {
+    printf( "Error Tcl_Init\n") ;
 	return TCL_ERROR;
     }
 #ifdef HAVE_ITCL
